/**
 * @file era_structures.h
 * @author Copyright (c) 2016 ESET, spol. s r.o.
 */

/**
 * Some structure definitions are deprecated and replaced by forward declarations and Get/Set functions. Use ERAAPI_REMOVE_DEPRECATED_SINCE_VERSION_5_3 before including era_structures.h to remove the definitions.
 */

#pragma once

//--------------------------
// ClientList
//--------------------------

struct Era_ClientList;

/** Allocates the structure and sets a default value for each member.
 * Has to be deleted using era_ClientList_delete.
 * @return the created structure pointer
 */
Era_ClientList * _stdcall era_ClientList_new(void);

/** Deallocates the structure and all its members.
 * @param list [in] structure to delete
 */
void _stdcall era_ClientList_delete(Era_ClientList *list);

/** Sets client selection to include all clients.
 * @param list [in] client list structure
 */
void _stdcall era_ClientList_SetAllClients(Era_ClientList *list);

/** Adds a client ID to the list.
 * @param clientId [in] client ID to add
 * @param list [in] client list structure
 */
void _stdcall era_ClientList_AddClient(Era_ClientList *list, const uint32_t clientId);

//--------------------------
// LicenseInfo
//--------------------------

#ifdef ERAAPI_REMOVE_DEPRECATED_SINCE_VERSION_5_3
struct Era_License;
#else
struct Era_License
{
	wchar_t customerName[255 + 1];
	uint32_t userCount;
	int64_t expirationTime;
	wchar_t productName[128 + 1];
	uint32_t id;
	uint32_t productId;
	wchar_t distGuid[128 + 1];
};
#endif

/** Gets customer name from a license.
 * @param license [in] license structure
 * @return customer name
 */
const wchar_t * _stdcall era_License_GetCustomerName(const Era_License *license);

/** Gets user count from a license.
 * @param license [in] license structure
 * @return user count
 */
uint32_t _stdcall era_License_GetUserCount(const Era_License *license);

/** Gets expiration time from a license.
 * @param license [in] license structure
 * @return expiration time
 */
int64_t _stdcall era_License_GetExpirationTime(const Era_License *license);

/** Gets product name from a license.
 * @param license [in] license structure
 * @return product name
 */
const wchar_t * _stdcall era_License_GetProductName(const Era_License *license);

/** Gets ID from a license.
 * @param license [in] license structure
 * @return license ID
 */
uint32_t _stdcall era_License_GetId(const Era_License *license);

/** Gets product ID from a license.
 * @param license [in] license structure
 * @return product ID
 */
uint32_t _stdcall era_License_GetProductId(const Era_License *license);

/** Gets partner ID from a license.
 * @param license [in] license structure
 * @return partner ID
 */
const wchar_t * _stdcall era_License_GetDistGuid(const Era_License *license);

#ifdef ERAAPI_REMOVE_DEPRECATED_SINCE_VERSION_5_3
struct Era_LicenseInfo;
#else
struct Era_LicenseInfo
{
	wchar_t customerName[255 + 1];
	uint32_t totalCount;
	int64_t expirationTime;
	bool licenseExpired;
	bool evalVersion;
	Era_License *licenses;
	uint32_t licenses_count;
};
#endif

/** Deallocates the structure and all its members (including Era_License items).
 * @param licenseInfo [in] structure to delete
 */
void _stdcall era_LicenseInfo_delete(Era_LicenseInfo *licenseInfo);

/** Gets customer name from a license.
 * @param licenseInfo [in] license info structure
 * @return customer name
 */
const wchar_t * _stdcall era_LicenseInfo_GetCustomerName(const Era_LicenseInfo *licenseInfo);

/** Gets total number of clients from a license.
 * @param licenseInfo [in] license info structure
 * @return total numer of clients
 */
uint32_t _stdcall era_LicenseInfo_GetTotalCount(const Era_LicenseInfo *licenseInfo);

/** Gets expiration time from a license.
 * @param licenseInfo [in] license info structure
 * @return expiration time
 */
int64_t _stdcall era_LicenseInfo_GetExpirationTime(const Era_LicenseInfo *licenseInfo);

/** Gets if the license is expired.
 * @param licenseInfo [in] license info structure
 * @return if the license is expired
 */
bool _stdcall era_LicenseInfo_IsLicenseExpired(const Era_LicenseInfo *licenseInfo);

/** Gets if the license is an evaluation license.
 * @param licenseInfo [in] license info structure
 * @return if the license is an evaluation license
 */
bool _stdcall era_LicenseInfo_IsEvalVersion(const Era_LicenseInfo *licenseInfo);

/** Gets license item of a license. Use era_LicenseInfo_GetLicensesCount to get items count.
 * @param licenseInfo [in] license info structure
 * @param itemIndex [in] item index
 * @return license item
 */
const Era_License * _stdcall era_LicenseInfo_GetLicense(const Era_LicenseInfo *licenseInfo, const uint32_t itemIndex);

/** Gets count of the license items contained in the license.
 * @param licenseInfo [in] license info structure
 * @return count of the license items
 */
uint32_t _stdcall era_LicenseInfo_GetLicensesCount(const Era_LicenseInfo *licenseInfo);

//--------------------------
// LicenseFiles
//--------------------------

#ifdef ERAAPI_REMOVE_DEPRECATED_SINCE_VERSION_5_3
struct Era_LicenseFiles;
#else
struct Era_LicenseFiles
{
	const wchar_t **fileNames;
	uint32_t fileNames_count;
};
#endif

/** Allocates the structure and sets a default value for each member.
 * Has to be deleted using era_LicenseFiles_delete.
 * @return the created structure pointer
 */
Era_LicenseFiles * _stdcall era_LicenseFiles_new(void);

/** Deallocates the structure and all its members.
 * @param files [in] structure to delete
 */
void _stdcall era_LicenseFiles_delete(Era_LicenseFiles *files);

/** Adds a name (path) of a license file.
 * @param files [in] license files structure
 * @param fileName [in] file path
 */
void _stdcall era_LicenseFiles_AddFile(Era_LicenseFiles *files, const wchar_t *fileName);


//--------------------------
// Query
//--------------------------

enum Era_Sign
{
	ERA_SIGN__EQUALS,
	ERA_SIGN__NOT_EQUALS,
	ERA_SIGN__GREATER,
	ERA_SIGN__LESS,
	ERA_SIGN__GREATER_EQUAL,
	ERA_SIGN__LESS_EQUAL,
	ERA_SIGN__LIKE,
	ERA_SIGN__NOTLIKE,
};

struct Era_Query;

/** Allocates the structure and sets a default value for each member.
 * Has to be deleted using era_Query_delete.
 * @return the created structure pointer
 */
Era_Query * _stdcall era_Query_new(void);

/** Deallocates the structure and all its members.
 * @param query [in] structure to delete
 */
void _stdcall era_Query_delete(Era_Query *query);

/** Sets a table to select data from.
 * @param query [in] query structure
 * @param tableName [in] name of the table
 */
void _stdcall era_Query_SetTable(Era_Query *query, const wchar_t *tableName);

/** Adds a column to get data for.
 * @param query [in] query structure
 * @param columnName [in] name of the column
 */
void _stdcall era_Query_AddColumn(Era_Query *query, const wchar_t *columnName);

/** Specifies that all columns have to be retrieved.
 * @param query [in] query structure
 */
void _stdcall era_Query_SetAllColumns(Era_Query *query);

/** Adds a condition to the query.
 * All such conditions will be connected using AND operator.
 * Use era_Query_AddConditionAlternative to add an alternative to the condition (connected using OR operator).
 * @param query [in] query structure
 * @param columnName [in] column name (data in this column will be compared in the condition)
 * @param sign [in] condition operator
 * @param value [in] constant value to compare with
 */
void _stdcall era_Query_AddCondition(Era_Query *query, const wchar_t *columnName, const Era_Sign sign, const wchar_t *value);

/** Adds an alternative condition to the last condition added using era_Query_addCondition.
 * The last condition added using era_Query_addCondition and all its alternatives (added by following calls to addConditionToAlternatives) will be forming a group of conditions connected using OR operator.
 * All such groups will be connected using AND operator.
 * @param query [in] query structure
 * @param columnName [in] column name (data in this column will be compared in the condition)
 * @param sign [in] condition operator
 * @param value [in] constant value to compare with
 */
void _stdcall era_Query_AddConditionAlternative(Era_Query *query, const wchar_t *columnName, const Era_Sign sign, const wchar_t *value);

/** Adds a column to group the data by.
 * The rows containing equal data in all columns added using era_Query_addGroupBy will be shown only once.
 * An additional column will be added which will contain the count of rows in each group.
 * @param query [in] query structure
 * @param columnName [in] column name to group by
 */
void _stdcall era_Query_AddGroupBy(Era_Query *query, const wchar_t *columnName);

/** Adds a column to order the result by.
 * More columns can be added. The following columns will be used if there are equal data in all the preceding columns.
 * @param query [in] query structure
 * @param columnName [in] column name to sort by
 * @param descending [in] sort direction (false for ascending, true for descending)
 */
void _stdcall era_Query_AddOrderBy(Era_Query *query, const wchar_t *columnName, const bool descending);

/** Sets a count of rows to skip in the beginning.
 * Use era_Query_SetLimit together with this function to specify the range of rows to show.
 * @param query [in] query structure
 * @param skip [in] count of rows to skip
 */
void _stdcall era_Query_SetSkip(Era_Query *query, const uint64_t skip);

/** Sets a count of rows to show.
 * Use era_Query_SetSkip together with this function to specify the range of rows to show.
 * @param query [in] query structure
 * @param limit [in] count of rows to show
 */
void _stdcall era_Query_SetLimit(Era_Query *query, const uint64_t limit);

//--------------------------
// TableRow
//--------------------------

#ifdef ERAAPI_REMOVE_DEPRECATED_SINCE_VERSION_5_3
struct Era_TableRow;
#else
struct Era_TableRow
{
	wchar_t **fields;
	uint32_t fields_count;
	wchar_t **prettyFields;
	uint32_t prettyFields_count;
};
#endif

/** Deallocates the structure and all its members.
 * @param tableRow [in] structure to delete
 */
void _stdcall era_TableRow_delete(Era_TableRow *tableRow);

/** Gets table row field as a string value. Use era_TableRow_GetFieldsCount to get fields count.
 * @param tableRow [in] table row structure
 * @param itemIndex [in] index of the field to get
 * @return field value
 */
const wchar_t * _stdcall era_TableRow_GetField(const Era_TableRow *tableRow, const uint32_t itemIndex);

/** Gets count of fields in the row.
 * @param tableRow [in] table row structure
 * @return count of fields
 */
uint32_t _stdcall era_TableRow_GetFieldsCount(const Era_TableRow *tableRow);

/** Gets table row field as a formatted (readable) string value. Use era_TableRow_GetPrettyFieldsCount to get fields count.
 * @param tableRow [in] table row structure
 * @param itemIndex [in] index of the field to get
 * @return pretty field value
 */
const wchar_t * _stdcall era_TableRow_GetPrettyField(const Era_TableRow *tableRow, const uint32_t itemIndex);

/** Gets count of pretty fields in the row.
 * @param tableRow [in] table row structure
 * @return count of fields
 */
uint32_t _stdcall era_TableRow_GetPrettyFieldsCount(const Era_TableRow *tableRow);

//--------------------------
// TableReader
//--------------------------

enum Era_Type
{
	ERA_TYPE__TEXT,
	ERA_TYPE__NUMBER,
	ERA_TYPE__BOOL,
	ERA_TYPE__IP_ADDRESS,
	ERA_TYPE__VERSION,
	ERA_TYPE__DATETIME,
	ERA_TYPE__DATA_STATE,
	ERA_TYPE__LOGLEVEL,
	ERA_TYPE__SCANTYPE,
	ERA_TYPE__BLOB_STATE,
	ERA_TYPE__SCHEDULER_FREQUENCY,
	ERA_TYPE__TASK_STATE,
	ERA_TYPE__TASK_TYPE,
	ERA_TYPE__TASK_CLIENT_STATE,
	ERA_TYPE__RITYPE,
	ERA_TYPE__RISTATE,
	ERA_TYPE__RIDIAG,
	ERA_TYPE__MAC_ADDRESS,
	ERA_TYPE__GROUP_TYPE,
	ERA_TYPE__POLICY_OVERRIDE,
};


#ifdef ERAAPI_REMOVE_DEPRECATED_SINCE_VERSION_5_3
struct Era_ColumnInfo;
#else
struct Era_ColumnInfo
{
	wchar_t *header;
	wchar_t *header_pretty;
	Era_Type type;
	wchar_t *typeName;
	wchar_t **typeValues;
	uint32_t typeValues_count;
	bool multipleValues;
};
#endif

/** Gets column header text
 * @param columnInfo [in] column info structure
 * @return header text
 */
const wchar_t * _stdcall era_ColumnInfo_GetHeader(const Era_ColumnInfo *columnInfo);

/** Gets formatted (readable) column header text
 * @param columnInfo [in] column info structure
 * @return formattedheader text
 */
const wchar_t * _stdcall era_ColumnInfo_GetPrettyHeader(const Era_ColumnInfo *columnInfo);

/** Gets column data type.
 * @param columnInfo [in] column info structure
 * @return data type
 */
Era_Type _stdcall era_ColumnInfo_GetType(const Era_ColumnInfo *columnInfo);

/** Gets column data type name as text.
 * @param columnInfo [in] column info structure
 * @return data type name
 */
const wchar_t * _stdcall era_ColumnInfo_GetTypeName(const Era_ColumnInfo *columnInfo);

/** Gets column possible value. Use era_ColumnInfo_GetTypeValuesCount to get values count.
 * @param columnInfo [in] column info structure
 * @param itemIndex [in] index of item to get
 * @return value
 */
const wchar_t * _stdcall era_ColumnInfo_GetTypeValue(const Era_ColumnInfo *columnInfo, const uint32_t itemIndex);

/** Gets count of possible values for the column.
 * @param columnInfo [in] column info structure
 * @return count
 */
uint32_t _stdcall era_ColumnInfo_GetTypeValuesCount(const Era_ColumnInfo *columnInfo);

/** Gets if the column can contain multiple values.
 * @param columnInfo [in] column info structure
 * @return if the column can contain multiple values
 */
bool _stdcall era_ColumnInfo_IsMultipleValues(const Era_ColumnInfo *columnInfo);

#ifdef ERAAPI_REMOVE_DEPRECATED_SINCE_VERSION_5_3
struct Era_TableReader;
#else
struct Era_TableReader
{
	Era_ColumnInfo **columns;
	uint32_t columns_count;

	uint64_t limit;
	bool limit_set;
	uint64_t nextRowIndex;
};
#endif

/** Deallocates the structure and all its members (including Era_ColumnInfo items).
 * @param tableReader [in] structure to delete
 */
void _stdcall era_TableReader_delete(Era_TableReader *tableReader);

/** Gets column info for the specified column. Use era_TableReader_GetColumnsCount to get columns count.
 * @param tableReader [in] table reader structure
 * @param itemIndex [in] index of the column to get info for
 * @return column info
 */
const Era_ColumnInfo * _stdcall era_TableReader_GetColumn(const Era_TableReader *tableReader, const uint32_t itemIndex);

/** Gets columns count.
 * @param tableReader [in] table reader structure
 * @return count
 */
uint32_t _stdcall era_TableReader_GetColumnsCount(const Era_TableReader *tableReader);

//--------------------------
// Query_Policy
//--------------------------

enum Era_PolicyDisplayType
{
	ERA_POLICY_DISPLAY_TYPE__TABLE,
	ERA_POLICY_DISPLAY_TYPE__TREE,
};

struct Era_PolicyQuery;

/** Allocates the structure and sets a default value for each member.
 * Has to be deleted using @c era_PolicyQuery_delete.
 * @return the created structure pointer
 */
Era_PolicyQuery * _stdcall era_PolicyQuery_new(void);

/** Deallocates the structure and all its members.
 * @param policyQuery [in] structure to delete
 */
void _stdcall era_PolicyQuery_delete(Era_PolicyQuery *policyQuery);

/** Sets a display type for the policy query.
 * @param policyQuery [in] policy query to set the display type to
 * @param displayType [in] display type to set to @c policyQuery
 */
void _stdcall era_PolicyQuery_SetDisplayType(Era_PolicyQuery *policyQuery, const Era_PolicyDisplayType displayType);

//--------------------------
// PolicyTableReader
//--------------------------

#ifdef ERAAPI_REMOVE_DEPRECATED_SINCE_VERSION_5_3
struct Era_PolicyTableReader;
#else
struct Era_PolicyTableReader
{
	Era_TableReader *tableReader;

	wchar_t *serverName;
	Era_PolicyDisplayType displayType;
};
#endif

/** Deallocates the structure and all its members.
 * @param policyTableReader [in] structure to delete
 */
void _stdcall era_PolicyTableReader_delete(Era_PolicyTableReader *policyTableReader);

/** Gets table reader (can be used to get info about columns).
 * @param policyTableReader [in] policy table reader structure
 * @return table reader
 */
const Era_TableReader * _stdcall era_PolicyTableReader_GetTableReader(const Era_PolicyTableReader *policyTableReader);

//--------------------------
// Query_Report
//--------------------------

enum Era_ReportType
{
	ERA_REPORT_TYPE__STATIC,
	ERA_REPORT_TYPE__DASHBOARD,
	ERA_REPORT_TYPE__GENERATED,
};

struct Era_ReportQuery;

/** Allocates the structure and sets a default value for each member.
 * Has to be deleted using @c era_ReportQuery_delete.
 * @return the created structure pointer
 */
Era_ReportQuery * _stdcall era_ReportQuery_new(void);

/** Deallocates the structure and all its members.
 * @param reportQuery [in] structure to delete
 */
void _stdcall era_ReportQuery_delete(Era_ReportQuery *reportQuery);

/** Sets a type for the report query.
 * @param reportQuery [in] report query to set the type to
 * @param type [in] type to set to @c reportQuery
 */
void _stdcall era_ReportQuery_SetType(Era_ReportQuery *reportQuery, const Era_ReportType type);

//--------------------------
// Report
//--------------------------

enum Era_ReportOutput
{
	ERA_REPORT_OUTPUT__HTML,
	ERA_REPORT_OUTPUT__ZIP,
	ERA_REPORT_OUTPUT__PDF,
};

//--------------------------
// ReportTemplateParams
//--------------------------

enum Era_ReportTemplateFlagState
{
	ERA_REPORT_TEMPLATE_FLAG_STATE__SET,
	ERA_REPORT_TEMPLATE_FLAG_STATE__UNSET,
	ERA_REPORT_TEMPLATE_FLAG_STATE__NOCHANGE,
};

struct Era_ReportTemplateParams;

/** Allocates the structure and sets a default value for each member.
 * Has to be deleted using @c era_ReportTemplateParams_delete.
 * @return the created structure pointer
 */
Era_ReportTemplateParams * _stdcall era_ReportTemplateParams_new(void);

/** Deallocates the structure and all its members.
 * @param params [in] structure to delete
 */
void _stdcall era_ReportTemplateParams_delete(Era_ReportTemplateParams *params);

/** Sets report template type (static or dashboard). Do not set when updating the report template, report type cannot be changed by update.
 * @param params [in] report template params structure
 * @param dashboard [in] report template type (false for static, true for dashboard)
 */
void _stdcall era_ReportTemplateParams_SetType(Era_ReportTemplateParams *params, const bool dashboard);

/** Sets report template name.
 * @param params [in] report template params structure
 * @param name [in] report template name
 */
void _stdcall era_ReportTemplateParams_SetName(Era_ReportTemplateParams *params, const wchar_t *name);

/** Sets report template configuration XML file.
 * @param params [in] report template params structure
 * @param configXml [in] report template configuration XML file
 */
void _stdcall era_ReportTemplateParams_SetConfigXml(Era_ReportTemplateParams *params, const wchar_t *configXml);

/** Sets report template active flag.
 * @param params [in] report template params structure
 * @param state [in] report template active flag value (set, unset, do no change)
 */
void _stdcall era_ReportTemplateParams_SetActive(Era_ReportTemplateParams *params, const Era_ReportTemplateFlagState state);

/** Sets report template description.
 * @param params [in] report template params structure
 * @param description [in] report template description
 */
void _stdcall era_ReportTemplateParams_SetDescription(Era_ReportTemplateParams *params, const wchar_t *description);

//--------------------------
// Query_Group
//--------------------------

enum Era_GroupDisplayType
{
	ERA_GROUP_DISPLAY_TYPE__TABLE,
	ERA_GROUP_DISPLAY_TYPE__TREE,
};

struct Era_GroupQuery;

/** Allocates the structure and sets a default value for each member.
 * Has to be deleted using @c era_GroupQuery_delete.
 * @return the created structure pointer
 */
Era_GroupQuery * _stdcall era_GroupQuery_new(void);

/** Deallocates the structure and all its members.
 * @param groupQuery [in] structure to delete
 */
void _stdcall era_GroupQuery_delete(Era_GroupQuery *groupQuery);

/** Sets a display type for the group query.
 * @param groupQuery [in] group query to set the display type to
 * @param displayType [in] display type to set to @c groupQuery
 */
void _stdcall era_GroupQuery_SetDisplayType(Era_GroupQuery *groupQuery, const Era_GroupDisplayType displayType);

//--------------------------
// GroupTableReader
//--------------------------

#ifdef ERAAPI_REMOVE_DEPRECATED_SINCE_VERSION_5_3
struct Era_GroupTableReader;
#else
struct Era_GroupTableReader
{
	Era_TableReader *tableReader;

	Era_GroupDisplayType displayType;
};
#endif

/** Deallocates the structure and all its members.
 * @param groupTableReader [in] structure to delete
 */
void _stdcall era_GroupTableReader_delete(Era_GroupTableReader *groupTableReader);

/** Gets table reader (can be used to get info about columns).
 * @param groupTableReader [in] group table reader structure
 * @return table reader
 */
const Era_TableReader * _stdcall era_GroupTableReader_GetTableReader(const Era_GroupTableReader *groupTableReader);

//--------------------------
// ScanParams
//--------------------------

enum Era_ScanProfile
{
	ERA_SCAN_PROFILE__IN_DEPTH_SCAN,
	ERA_SCAN_PROFILE__MY_PROFILE,
	ERA_SCAN_PROFILE__SMART_SCAN,
	ERA_SCAN_PROFILE__CONTEXT_MENU_SCAN,
};

enum Era_Target_Windows
{
	ERA_TARGET_WINDOWS__MEMORY,
	ERA_TARGET_WINDOWS__REMOVABLE_DRIVES_BOOT,
	ERA_TARGET_WINDOWS__REMOVABLE_DRIVES,
	ERA_TARGET_WINDOWS__LOCAL_DRIVES_BOOT,
	ERA_TARGET_WINDOWS__LOCAL_DRIVES,
	ERA_TARGET_WINDOWS__REMOTE_DRIVES,
	ERA_TARGET_WINDOWS__ALL_DRIVES_BOOT,
	ERA_TARGET_WINDOWS__ALL_DRIVES,
};

struct Era_ScanParams_Windows;

/** Allocates the structure and sets a default value for each member.
 * Has to be deleted using era_ScanParams_Windows_delete.
 * @return the created structure pointer
 */
Era_ScanParams_Windows * _stdcall era_ScanParams_Windows_new(void);

/** Deallocates the structure and all its members.
 * @param params [in] structure to delete
 */
void _stdcall era_ScanParams_Windows_delete(Era_ScanParams_Windows *params);

/** Sets the scan pofile.
 * Use era_ScanParams_Windows_SetProfile_userDefined to sets a user-defined profile name.
 * @param params [in] Windows scan params structure
 * @param profile [in] profile to set
 */
void _stdcall era_ScanParams_Windows_SetProfile(Era_ScanParams_Windows *params, const Era_ScanProfile profile);

/** Sets an user-defined scan pofile.
 * Use era_ScanParams_Windows_SetProfile to select from predefined profiles.
 * @param params [in] Windows scan params structure
 * @param profileName [in] name of profile to set
 */
void _stdcall era_ScanParams_Windows_SetProfile_userDefined(Era_ScanParams_Windows *params, const wchar_t *profileName);

/** Adds a scan target.
 * Use era_ScanParams_Windows_AddTarget_userDefinedPath to add an user-defined path.
 * @param params [in] Windows scan params structure
 * @param target [in] scan target to add
 */
void _stdcall era_ScanParams_Windows_AddTarget(Era_ScanParams_Windows *params, const Era_Target_Windows target);

/** Adds an user-defined path as scan target.
 * Use era_ScanParams_Windows_AddTarget to select from predefined targets.
 * @param params [in] Windows scan params structure
 * @param path [in] path to add
 */
void _stdcall era_ScanParams_Windows_AddTarget_userDefinedPath(Era_ScanParams_Windows *params, const wchar_t *path);

/** Sets no cleaning mode.
 * @param params [in] Windows scan params structure
 */
void _stdcall era_ScanParams_Windows_SetNoCleaning(Era_ScanParams_Windows *params);

/** Sets to shutdown computer after scan.
 * Use era_ScanParams_Windows_SetAllowShutdownCancel to allow shutdown canceling.
 * @param params [in] Windows scan params structure
 */
void _stdcall era_ScanParams_Windows_SetShutdownComputerAfterScan(Era_ScanParams_Windows *params);

/** Sets allowing canceling of shutdown.
 * Use era_ScanParams_Windows_SetShutdownComputerAfterScan to set computer shutdown before setting this.
 * @param params [in] Windows scan params structure
 */
void _stdcall era_ScanParams_Windows_SetAllowShutdownCancel(Era_ScanParams_Windows *params);

struct Era_ScanParams_Linux3;

/** Allocates the structure and sets a default value for each member.
 * Has to be deleted using era_ScanParams_Linux3_delete.
 * @return the created structure pointer
 */
Era_ScanParams_Linux3 * _stdcall era_ScanParams_Linux3_new(void);

/** Deallocates the structure and all its members.
 * @param params [in] structure to delete
 */
void _stdcall era_ScanParams_Linux3_delete(Era_ScanParams_Linux3 *params);

/** Adds an user-defined path as scan target.
 * @param params [in] Linux3 scan params structure
 * @param path [in] path to add
 */
void _stdcall era_ScanParams_Linux3_AddTarget_userDefinedPath(Era_ScanParams_Linux3 *params, const wchar_t *path);

/** Sets no cleaning mode.
 * @param params [in] Linux3 scan params structure
 */
void _stdcall era_ScanParams_Linux3_SetNoCleaning(Era_ScanParams_Linux3 *params);

struct Era_ScanParams_Linux;

/** Allocates the structure and sets a default value for each member.
 * Has to be deleted using era_ScanParams_Linux_delete.
 * @return the created structure pointer
 */
Era_ScanParams_Linux * _stdcall era_ScanParams_Linux_new(void);

/** Deallocates the structure and all its members.
 * @param params [in] structure to delete
 */
void _stdcall era_ScanParams_Linux_delete(Era_ScanParams_Linux *params);

/** Sets the scan pofile.
 * Use era_ScanParams_Linux_SetProfile_userDefined to sets a user-defined profile name.
 * @param params [in] Linux scan params structure
 * @param profile [in] profile to set
 */
void _stdcall era_ScanParams_Linux_SetProfile(Era_ScanParams_Linux *params, const Era_ScanProfile profile);

/** Sets an user-defined scan pofile.
 * Use era_ScanParams_Linux_SetProfile to select from predefined profiles.
 * @param params [in] Linux scan params structure
 * @param profileName [in] name of profile to set
 */
void _stdcall era_ScanParams_Linux_SetProfile_userDefined(Era_ScanParams_Linux *params, const wchar_t *profileName);

/** Adds an user-defined path as scan target.
 * @param params [in] Linux scan params structure
 * @param path [in] path to add
 */
void _stdcall era_ScanParams_Linux_AddTarget_userDefinedPath(Era_ScanParams_Linux *params, const wchar_t *path);

/** Sets no cleaning mode.
 * @param params [in] Linux scan params structure
 */
void _stdcall era_ScanParams_Linux_SetNoCleaning(Era_ScanParams_Linux *params);

enum Era_Target_Mobile
{
	ERA_TARGET_MOBILE__ALL,
};

struct Era_ScanParams_Mobile;

/** Allocates the structure and sets a default value for each member.
 * Has to be deleted using era_ScanParams_Mobile_delete.
 * @return the created structure pointer
 */
Era_ScanParams_Mobile * _stdcall era_ScanParams_Mobile_new(void);

/** Deallocates the structure and all its members.
 * @param params [in] structure to delete
 */
void _stdcall era_ScanParams_Mobile_delete(Era_ScanParams_Mobile *params);

/** Adds a scan target.
 * Use era_ScanParams_Mobile_AddTarget_userDefinedPath to add an user-defined path.
 * @param params [in] mobile scan params structure
 * @param target [in] scan target to add
 */
void _stdcall era_ScanParams_Mobile_AddTarget(Era_ScanParams_Mobile *params, const Era_Target_Mobile target);

/** Adds an user-defined path as scan target.
 * Use era_ScanParams_Mobile_AddTarget to select from predefined targets.
 * @param params [in] mobile scan params structure
 * @param path [in] path to add
 */
void _stdcall era_ScanParams_Mobile_AddTarget_userDefinedPath(Era_ScanParams_Mobile *params, const wchar_t *path);

/** Sets no cleaning mode.
 * @param params [in] mobile scan params structure
 */
void _stdcall era_ScanParams_Mobile_SetNoCleaning(Era_ScanParams_Mobile *params);

//--------------------------
// ServerInfo
//--------------------------

#ifdef ERAAPI_REMOVE_DEPRECATED_SINCE_VERSION_5_3
struct Era_ServerInfo;
#else
struct Era_ServerInfo
{
	wchar_t *serverName;
	uint32_t version_major;
	uint32_t version_minor;
	uint32_t version_build;
};
#endif

/** Deallocates the structure and all its members.
 * @param serverInfo [in] structure to delete
 */
void _stdcall era_ServerInfo_delete(Era_ServerInfo *serverInfo);

/** Gets server name from server info.
 * @param serverInfo [in] server info structure
 * @return server name
 */
const wchar_t * _stdcall era_ServerInfo_GetServerName(const Era_ServerInfo *serverInfo);

/** Gets major version from server info.
 * @param serverInfo [in] server info structure
 * @return major version
 */
uint32_t _stdcall era_ServerInfo_GetMajorVersion(const Era_ServerInfo *serverInfo);

/** Gets minor version from server info.
 * @param serverInfo [in] server info structure
 * @return minor version
 */
uint32_t _stdcall era_ServerInfo_GetMinorVersion(const Era_ServerInfo *serverInfo);

/** Gets build version from server info.
 * @param serverInfo [in] server info structure
 * @return build version
 */
uint32_t _stdcall era_ServerInfo_GetBuildVersion(const Era_ServerInfo *serverInfo);

//--------------------------
// SysInspectorParams
//--------------------------

#define ERA_DATA_CONFIGURATION 1
#define ERA_DATA_FEATURES 2
#define ERA_DATA_STATUS 4
#define ERA_DATA_SYSTEM_INFORMATION 8
#define ERA_DATA_SYSINSPECTOR 16
#define ERA_DATA_POLICY 32
#define ERA_DATA_POLICY_MERGED 64
#define ERA_DATA_POLICY_OVERRIDE 128
#define ERA_DATA_POLICY_NONOVERRIDE 256

struct Era_SysinspectorParams;

/** Allocates the structure and sets a default value for each member.
 * Has to be deleted using era_TaskParams_delete.
 * @return the created structure pointer
 */
Era_SysinspectorParams * _stdcall era_SysinspectorParams_new(void);

/** Deallocates the structure and all its members.
 * @param params [in] structure to delete
 */
void _stdcall era_SysinspectorParams_delete(Era_SysinspectorParams *params);

/** Sets sysinspector comparision date.
 * @param params [in] sysinspectors params structure
 * @param time [in] time to compare with
 */
void _stdcall era_SysinspectorParams_SetCompareToDate(Era_SysinspectorParams *params, const uint64_t time);

/** Sets sysinspector snapshot creation
 * @param params [in] sysinspectors params structure
 * @param createSnapshot [in] 
 */
void _stdcall era_SysinspectorParams_SetSnapshot(Era_SysinspectorParams *params, const bool createSnapshot);

//--------------------------
// TaskParams
//--------------------------

struct Era_TaskParams;

/** Allocates the structure and sets a default value for each member.
 * Has to be deleted using era_TaskParams_delete.
 * @return the created structure pointer
 */
Era_TaskParams * _stdcall era_TaskParams_new(void);

/** Deallocates the structure and all its members.
 * @param params [in] structure to delete
 */
void _stdcall era_TaskParams_delete(Era_TaskParams *params);

/** Sets task name.
 * @param params [in] task params structure
 * @param name [in] task name
 */
void _stdcall era_TaskParams_SetName(Era_TaskParams *params, const wchar_t *name);

/** Sets task description.
 * @param params [in] task params structure
 * @param description [in] task description
 */
void _stdcall era_TaskParams_SetDescription(Era_TaskParams *params, const wchar_t *description);

/** Sets the time when the task has to be applied.
 * @param params [in] task params structure
 * @param time [in] time when the task has to be applied
 */
void _stdcall era_TaskParams_SetApplyAfter(Era_TaskParams *params, const uint64_t time);

/** Sets task deleting after completion.
 * @param params [in] task params structure
 */
void _stdcall era_TaskParams_SetDeleteIfCompleted(Era_TaskParams *params);

//--------------------------
// TablesInfo
//--------------------------

#ifdef ERAAPI_REMOVE_DEPRECATED_SINCE_VERSION_5_3
struct Era_TablesInfo;
#else
struct Era_TablesInfo
{
	wchar_t **tableNames;
	uint32_t tableNames_count;
	wchar_t **prettyTableNames;
	uint32_t prettyTableNames_count;
};
#endif

/** Deallocates the structure and all its members.
 * @param tablesInfo [in] structure to delete
 */
void _stdcall era_TablesInfo_delete(Era_TablesInfo *tablesInfo);

/** Gets table name. Use era_TablesInfo_GetTableNamesCount to get count of table names.
 * @param tablesInfo [in] tables info structure
 * @param itemIndex [in] index of the item to get
 * @return table name
 */
const wchar_t * _stdcall era_TablesInfo_GetTableName(const Era_TablesInfo *tablesInfo, const uint32_t itemIndex);

/** Gets count of table names.
 * @param tablesInfo [in] tables info structure
 * @return count
 */
uint32_t _stdcall era_TablesInfo_GetTableNamesCount(const Era_TablesInfo *tablesInfo);

/** Gets formatted (readable) table name. Use era_TablesInfo_GetPrettyTableNamesCount to get pretty table names count.
 * @param tablesInfo [in] tables info structure
 * @param itemIndex [in] index of the item to get
 * @return table name
 */
const wchar_t * _stdcall era_TablesInfo_GetPrettyTableName(const Era_TablesInfo *tablesInfo, const uint32_t itemIndex);

/** Gets pretty table names count.
 * @param tablesInfo [in] tables info structure
 * @return count
 */
uint32_t _stdcall era_TablesInfo_GetPrettyTableNamesCount(const Era_TablesInfo *tablesInfo);

//--------------------------
// PolicyParams
//--------------------------

enum Era_PolicyFlagState
{
	ERA_POLICY_FLAG_STATE__SET,
	ERA_POLICY_FLAG_STATE__UNSET,
	ERA_POLICY_FLAG_STATE__NOCHANGE,
};

struct Era_PolicyParams;

/** Allocates the structure and sets a default value for each member.
 * Has to be deleted using @c era_PolicyParams_delete.
 * @return the created structure pointer
 */
Era_PolicyParams * _stdcall era_PolicyParams_new(void);

/** Deallocates the structure and all its members.
 * @param params [in] structure to delete
 */
void _stdcall era_PolicyParams_delete(Era_PolicyParams *params);

/** Sets policy name.
 * @param params [in] policy params structure
 * @param name [in] policy name
 */
void _stdcall era_PolicyParams_SetName(Era_PolicyParams *params, const wchar_t *name);

/** Sets policy parent ID.
 * @param params [in] policy params structure
 * @param parentId [in] policy parent ID
 */
void _stdcall era_PolicyParams_SetParentId(Era_PolicyParams *params, const uint32_t parentId);

/** Sets policy parent to no parent policy.
 * @param params [in] policy params structure
 */
void _stdcall era_PolicyParams_SetNoParent(Era_PolicyParams *params);

/** Sets policy configuration XML file.
 * @param params [in] policy params structure
 * @param configXml [in] policy configuration XML file
 */
void _stdcall era_PolicyParams_SetConfigXml(Era_PolicyParams *params, const wchar_t *configXml);

/** Sets policy description.
 * @param params [in] policy params structure
 * @param description [in] policy description
 */
void _stdcall era_PolicyParams_SetDescription(Era_PolicyParams *params, const wchar_t *description);

/** Sets policy flag - override any child policy.
 * @param params [in] policy params structure
 * @param state [in] override any child policy flag state (set, unset, do no change)
 */
void _stdcall era_PolicyParams_OverrideAnyChild(Era_PolicyParams *params, const Era_PolicyFlagState state);

/** Sets policy flag - down repliceable policy.
 * @param params [in] policy params structure
 * @param state [in] down repliceable policy flag state (set, unset, do no change)
 */
void _stdcall era_PolicyParams_DownReplicable(Era_PolicyParams *params, const Era_PolicyFlagState state);

/** Sets policy flag - default for clients.
 * @param params [in] policy params structure
 * @param state [in] default for clients flag state (set, unset, do no change)
 */
void _stdcall era_PolicyParams_SetSetAsDefaultForClients(Era_PolicyParams *params, const Era_PolicyFlagState state);

/** Sets policy flag - default for lower servers.
 * @param params [in] policy params structure
 * @param state [in] default for lower servers flag state (set, unset, do no change)
 */
void _stdcall era_PolicyParams_SetSetAsDefaultForLowerServers(Era_PolicyParams *params, const Era_PolicyFlagState state);

//--------------------------
// PolicyReplacement
//--------------------------

enum Era_PolicyReplacementSelection
{
	ERA_POLICY_REPLACEMENT_SELECTION__NOT_SELECTED,
	ERA_POLICY_REPLACEMENT_SELECTION__DEFAULT_CLIENTS_POLICY,
	ERA_POLICY_REPLACEMENT_SELECTION__DEFAULT_UPPER_SERVER_POLICY,
	ERA_POLICY_REPLACEMENT_SELECTION__NOT_AVAILABLE,
	ERA_POLICY_REPLACEMENT_SELECTION__POLICY_ID,
};

struct Era_PolicyReplacement;

/** Allocates the structure and sets a default value for each member.
 * Has to be deleted using era_PolicyReplacement_delete.
 * @return the created structure pointer
 */
Era_PolicyReplacement * _stdcall era_PolicyReplacement_new(void);

/** Deallocates the structure and all its members.
 * @param replacement [in] structure to delete
 */
void _stdcall era_PolicyReplacement_delete(Era_PolicyReplacement *replacement);

/** Selects default clients policy as a replacement.
 * @param replacement [in] policy replacement structure
 */
void _stdcall era_PolicyReplacement_SelectDefaultClientsPolicy(Era_PolicyReplacement *replacement);

/** Selects default upper server policy as a replacement.
 * @param replacement [in] policy replacement structure
 */
void _stdcall era_PolicyReplacement_SelectDefaultUpperServerPolicy(Era_PolicyReplacement *replacement);

/** Selects the not available state as a replacement (the policy will be not specified after replacing).
 * @param replacement [in] policy replacement structure
 */
void _stdcall era_PolicyReplacement_SelectNotAvailable(Era_PolicyReplacement *replacement);

/** Selects the specific policy as a replacement.
 * @param replacement [in] policy replacement structure
 * @param policyId [in] policy ID to set as a replacement
 */
void _stdcall era_PolicyReplacement_SelectPolicy(Era_PolicyReplacement *replacement, const uint32_t policyId);

//--------------------------
// ServerConfigurationList
//--------------------------

enum Era_ServerCfgType
{
	ERA_SERVER_CFG_TYPE__NUMERIC_VALUE,
	ERA_SERVER_CFG_TYPE__STRING_VALUE,
	ERA_SERVER_CFG_TYPE__PASSWORD_VALUE,
};

#ifdef ERAAPI_REMOVE_DEPRECATED_SINCE_VERSION_5_3
struct Era_ServerConfigurationItem;
#else
struct Era_ServerConfigurationItem
{
	const wchar_t *name;
	const wchar_t *description;
	Era_ServerCfgType type;
};
#endif

/** Gets server configuration item name.
 * @param serverConfigurationItem [in] server configuration item structure
 * @return name
 */
const wchar_t * _stdcall era_ServerConfigurationItem_GetName(const Era_ServerConfigurationItem *serverConfigurationItem);

/** Gets server configuration item description.
 * @param serverConfigurationItem [in] server configuration item structure
 * @return description
 */
const wchar_t * _stdcall era_ServerConfigurationItem_GetDescription(const Era_ServerConfigurationItem *serverConfigurationItem);

/** Gets server configuration item type.
 * @param serverConfigurationItem [in] server configuration item structure
 * @return type
 */
Era_ServerCfgType _stdcall era_ServerConfigurationItem_GetType(const Era_ServerConfigurationItem *serverConfigurationItem);

#ifdef ERAAPI_REMOVE_DEPRECATED_SINCE_VERSION_5_3
struct Era_ServerConfigurationList;
#else
struct Era_ServerConfigurationList
{
	Era_ServerConfigurationItem *items;
	uint32_t items_count;
};
#endif

/** Deallocates the structure and all its members.
 * @param serverConfigurationList [in] structure to delete
 */
void _stdcall era_ServerConfigurationList_delete(Era_ServerConfigurationList *serverConfigurationList);

/** Gets configuration item from the list.
 * @param serverConfigurationList [in] server configuration list structure
 * @param itemIndex [in] index of the item to get
 * @return configuration item
 */
const Era_ServerConfigurationItem * _stdcall era_ServerConfigurationList_GetItem(const Era_ServerConfigurationList *serverConfigurationList, const uint32_t itemIndex);

/** Gets count of items in server configuration list.
 * @param serverConfigurationList [in] server configuration list structure
 * @return count
 */
uint32_t _stdcall era_ServerConfigurationList_GetItemsCount(const Era_ServerConfigurationList *serverConfigurationList);

//--------------------------
// ChangePassword
//--------------------------

enum Era_PasswordType
{
	ERA_PASSWORD_TYPE__INSTALLER,
	ERA_PASSWORD_TYPE__REPLICATION,
	ERA_PASSWORD_TYPE__CLIENT,
	ERA_PASSWORD_TYPE__CURRENTUSER,
};


//--------------------------
// EncryptPassword
//--------------------------

#ifdef ERAAPI_REMOVE_DEPRECATED_SINCE_VERSION_5_3
struct Era_EncryptedPassword;
#else
struct Era_EncryptedPassword
{
	wchar_t* encryptedPassword;
};
#endif

/** Deallocates the structure and all its members.
 * @param encryptedPassword [in] structure to delete
 */
void _stdcall era_EncryptedPassword_delete(Era_EncryptedPassword *encryptedPassword);

/** Gets encrypted password from encrypted password structure
 * @param encryptedPassword [in] encrypted password structure
 * @return encrypted password
 */
const wchar_t * _stdcall era_EncryptedPassword_GetEncryptedPassword(const Era_EncryptedPassword *encryptedPassword);

enum Era_PasswordEncryptionType
{		
	ERA_PASSWORD_ENCRYPTIONTYPE__SERVER,
	ERA_PASSWORD_ENCRYPTIONTYPE__OTHER,
	ERA_PASSWORD_ENCRYPTIONTYPE__HASH,
	ERA_PASSWORD_ENCRYPTIONTYPE__NUM,
};

//--------------------------
// LocalPathType
//--------------------------

enum Era_LocalPathType
{
	ERA_LOCAL_PATH_TYPE__INSTALL_DIR,
	ERA_LOCAL_PATH_TYPE__APPLICATION_DATA_DIR,
};


//--------------------------
// ScanLog
//--------------------------

struct Era_ScanLog;

//--------------------------
// LogForwardInfo
//--------------------------

#ifdef ERAAPI_REMOVE_DEPRECATED_SINCE_VERSION_5_3
struct Era_LogForward;
#else
struct Era_LogForward
{
	wchar_t enabled[64+1];
	wchar_t log[127+1];
	wchar_t level[127+1];
	wchar_t severity[127+1];
	wchar_t facility[255+1];
};
#endif

/** Gets enabled state if log forward.
 * @param logForward [in] log forward structure
 * @return enabled state
 */
const wchar_t * _stdcall era_LogForward_GetEnabled(const Era_LogForward *logForward);

/** Gets log from log forward.
 * @param logForward [in] log forward structure
 * @return log
 */
const wchar_t * _stdcall era_LogForward_GetLog(const Era_LogForward *logForward);

/** Gets level from log forward.
 * @param logForward [in] log forward structure
 * @return level
 */
const wchar_t * _stdcall era_LogForward_GetLevel(const Era_LogForward *logForward);

/** Gets severity from log forward.
 * @param logForward [in] log forward structure
 * @return severity
 */
const wchar_t * _stdcall era_LogForward_GetSeverity(const Era_LogForward *logForward);

/** Gets facility from log forward.
 * @param logForward [in] log forward structure
 * @return facility
 */
const wchar_t * _stdcall era_LogForward_GetFacility(const Era_LogForward *logForward);

#ifdef ERAAPI_REMOVE_DEPRECATED_SINCE_VERSION_5_3
struct Era_LogForwardInfo;
#else
struct Era_LogForwardInfo
{
	Era_LogForward *logForwards;
	uint32_t count;
};
#endif

/** Deallocates the structure and all its members.
 * @param logForwardInfo [in] structure to delete
 */
void _stdcall era_LogForwardInfo_delete(Era_LogForwardInfo *logForwardInfo);

/** Gets log forward item from log forward info. Use era_LogForwardInfo_GetLogForwardsCount to get count of log forward items.
 * @param logForwardInfo [in] log forward info structure
 * @param itemIndex [in] index of info to get
 * @return log forward item
 */
const Era_LogForward * _stdcall era_LogForwardInfo_GetLogForward(const Era_LogForwardInfo *logForwardInfo, const uint32_t itemIndex);

/** Gets count of log forward items.
 * @param logForwardInfo [in] log forward info structure
 * @return count
 */
uint32_t _stdcall era_LogForwardInfo_GetLogForwardsCount(const Era_LogForwardInfo *logForwardInfo);

//--------------------------
// PolicyRuleParams
//--------------------------

enum Era_PolicyRulePriority
{	
	ERA_POLICY_RULE_PRIORITY_TOP,
	ERA_POLICY_RULE_PRIORITY_BOTTOM,
	ERA_POLICY_RULE_PRIORITY_UP,		//not valid for era_PolicyRuleCreate
	ERA_POLICY_RULE_PRIORITY_DOWN,		//not valid for era_PolicyRuleCreate
	ERA_POLICY_RULE_PRIORITY_NOUPDATE,	//not valid for era_PolicyRuleCreate
};

struct Era_PolicyRuleParams;

/** Allocates the structure and sets a default value for each member. Default value means no updating when updating a rule. For creating a rule, all values must be initialized.
 * Has to be deleted using era_PolicyRuleData_delete.
 * @return the created structure pointer
 */
Era_PolicyRuleParams * _stdcall era_PolicyRuleParams_new(void);

/** Deallocates the structure and all its members.
 * @param ruleData [in] structure to delete
 */
void _stdcall era_PolicyRuleParams_delete(Era_PolicyRuleParams *ruleData);

/** Selects the policy for policy rule.
 * @param params [in] policy rule structure
 * @param policyId [in] policy ID to set
 */
void _stdcall era_PolicyRuleParams_SelectPolicyId(Era_PolicyRuleParams *params, const uint32_t policyId);

/** Sets XML file for creating policy rule
 * @param params [in] policy rule structure
 * @param ruleXml [in] rule XML file
 */
void _stdcall era_PolicyRuleParams_SetRuleXml(Era_PolicyRuleParams *params, const wchar_t *ruleXml);

/** Sets policy rule name.
 * @param params [in] policy rule structure
 * @param ruleName [in] policy rule name
 */
void _stdcall era_PolicyRuleParams_SetName(Era_PolicyRuleParams *params, const wchar_t *ruleName);

/** Sets policy rule description.
 * @param params [in] policy params structure
 * @param description [in] policy rule description
 */
void _stdcall era_PolicyRuleParams_SetDescription(Era_PolicyRuleParams *params, const wchar_t *description);

/** Sets policy rule priority (or priority update for rule update).
 * @param params [in] policy params structure
 * @param priority [in] policy rule priority
 */
void _stdcall era_PolicyRuleParams_SetPriority(Era_PolicyRuleParams *params, Era_PolicyRulePriority priority);

/** Sets policy rule state: enabled/disabled
 * @param params [in] policy params structure
 * @param enabled [in] policy rule enable state, 0 for false, 1 for true, -1 for unuchanged (used in update)
 */
void _stdcall era_PolicyRuleParams_SetEnableState(Era_PolicyRuleParams *params, int enabled);

//--------------------------
// GroupParams
//--------------------------

enum Era_GroupType
{
	ERA_GROUP_TYPE__STATIC,
	ERA_GROUP_TYPE__PARAMETRIC,
};

enum Era_GroupFlagState
{
	ERA_GROUP_FLAG_STATE__SET,
	ERA_GROUP_FLAG_STATE__UNSET,
	ERA_GROUP_FLAG_STATE__NOCHANGE,
};

struct Era_GroupParams;

/** Allocates the structure and sets a default value for each member.
 * Has to be deleted using @c era_GroupParams_delete.
 * @return the created structure pointer
 */
Era_GroupParams * _stdcall era_GroupParams_new(void);

/** Deallocates the structure and all its members.
 * @param params [in] structure to delete
 */
void _stdcall era_GroupParams_delete(Era_GroupParams *params);

/** Sets group type (static or parametric).
 * @param params [in] group params structure
 * @param type [in] group type
 */
void _stdcall era_GroupParams_SetType(Era_GroupParams *params, Era_GroupType type);

/** Sets group name.
 * @param params [in] group params structure
 * @param name [in] group name
 */
void _stdcall era_GroupParams_SetName(Era_GroupParams *params, const wchar_t *name);

/** Sets group parent ID.
 * @param params [in] group params structure
 * @param parentId [in] group parent ID
 */
void _stdcall era_GroupParams_SetParentId(Era_GroupParams *params, const uint32_t parentId);

/** Sets group parent to no parent group.
 * @param params [in] group params structure 
 */
void _stdcall era_GroupParams_SetNoParent(Era_GroupParams *params);

/** Sets parametric group configuration XML file. For parametric groups only. Not applicable to static groups.
 * @param params [in] group params structure
 * @param paramsXml [in] group parameters XML file
 */
void _stdcall era_GroupParams_SetParamsXml(Era_GroupParams *params, const wchar_t *paramsXml);

/** Sets group description.
 * @param params [in] group params structure
 * @param description [in] group description
 */
void _stdcall era_GroupParams_SetDescription(Era_GroupParams *params, const wchar_t *description);

/** Sets parametric group sticky flag. For parametric groups only. Not applicable to static groups.
 * Can only be set when creating a new group, ignored on update.
 * @param params [in] group params structure
 * @param sticky [in] sticky parametric group
 */
void _stdcall era_GroupParams_SetSticky(Era_GroupParams *params, const Era_GroupFlagState sticky);

//--------------------------
// ClearInfo
//--------------------------

#define ERA_INFO_CLEAR_LAST_THREAT		0x0001
#define ERA_INFO_CLEAR_LAST_EVENT		0x0002
#define ERA_INFO_CLEAR_LAST_FIREWALL	0x0004
#define ERA_INFO_CLEAR_LAST_SCANS		0x0008
#define ERA_INFO_CLEAR_CUSTOM_INFO		0x0100

//--------------------------
// RunNow
//--------------------------

#define ERA_RUN_NOW__CLEANUP 0x0001
#define ERA_RUN_NOW__COMPACT 0x0002
#define ERA_RUN_NOW__REPLICATE 0x0004
#define ERA_RUN_NOW__REPLICATE_WITH_MARK_ALL_CLIENTS 0x0008
#define ERA_RUN_NOW__UPDATE 0x0010
#define ERA_RUN_NOW__UPDATE_WITH_CLEAR_CACHE 0x0020
#define ERA_RUN_NOW__APPLY_POLICY_RULES 0x0040
#define ERA_RUN_NOW__SYNCHRONIZE_PARAMETRIC_GROUPS 0x0080

