/**
 * @file CommandParser.cpp
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "CommandParser.h"

#include <Common/era_api/err_codes.h>
#include "EracmdExceptions.h"
#include "Token.h"

struct CommandLineOperator
{
	LPCWSTR Text;
	Token::TokenType Type;
};

static CommandLineOperator Operators[] =
{
	{L"=", Token::token_equals},
	{L"!=", Token::token_notequals},
	{L"<=", Token::token_lessequal},
	{L">=", Token::token_greaterequal},
	{L"<", Token::token_less},
	{L">", Token::token_greater},
	{L"-EQ", Token::token_equals},
	{L"-NE", Token::token_notequals},
	{L"-LT", Token::token_less},
	{L"-GT", Token::token_greater},
	{L"-LE", Token::token_lessequal},
	{L"-GE", Token::token_greaterequal},
	{L"-LIKE", Token::token_like},
	{L"-NOTLIKE", Token::token_notlike},
	{L"-IN", Token::token_in},
	{L"-NOTIN", Token::token_notin},
	{L"-MEMEBEROF", Token::token_memberof},
	{L"-POLICYIS", Token::token_policyis},
	{L"-ASC", Token::token_asc},
	{L"-DESC", Token::token_desc},
};


bool CommandParser::CheckOperator(LPCWSTR *CommandLine, tokenArray &Tokens)
{
	for (size_t i = 0; i < _countof(Operators); ++i)
	{
		CommandLineOperator &op = Operators[i];
		size_t len = wcslen(op.Text);
		if (_wcsnicmp(*CommandLine, op.Text, len) == 0)
		{
			AddText(Tokens);
			Tokens.push_back(std::make_shared<Token>(std::wstring(*CommandLine).substr(0, len), op.Type));
			(*CommandLine)+= len;
			return true;
		}
	}
	return false;
}

// adds bufferred text
void CommandParser::AddText(tokenArray &Tokens)
{
	if (Text.empty())
		return;
	std::wstring word;
	LPCWSTR p = Text.c_str();
	WCHAR ch;
	bool inQuote = false;
	while ((ch = *p++) != NULL)
	{
		if (ch == L'"')
		{
			if (inQuote)
			{
				// doubled quote?
				if (*p == L'"')
				{
					p++;
				}
				else
				{
					inQuote = false;
					continue;
				}
			}
			else
			{
				inQuote = true;
				continue;
			}
		}
		word.push_back(ch);
	}
	while (ch);
	Tokens.push_back(std::make_shared<Token>(word, Token::token_text));
	Text.clear();
}

std::wstring CommandParser::ToStr(WCHAR ch)
{
	std::wstring res;
	res.push_back(ch);
	return res;
}

void CommandParser::ProcessCommandLine(LPCWSTR CommandLine, tokenArray &Tokens)
{
	Tokens.clear();
	Text.clear();
	WCHAR ch = L'\0';
	bool inQuote = false;
	do
	{
		if (!inQuote)
			if (CheckOperator(&CommandLine, Tokens))
				continue;

		ch = *CommandLine++;

		if (ch == '\0')
			break;

		// quoted text is out of processing
		if (ch == L'\"')
		{
			inQuote = !inQuote;
			Text.push_back(ch);
			continue;
		}
		if (!inQuote)
		{
			switch(ch)
			{
				case L'(':
					AddText(Tokens);
					Tokens.push_back(std::make_shared<Token>(ToStr(ch), Token::token_par_left));
					continue;
				case L')':
					AddText(Tokens);
					Tokens.push_back(std::make_shared<Token>(ToStr(ch), Token::token_par_right));
					continue;
				case L'\r':
				case L'\n':
					continue;
				case L',':
					AddText(Tokens);
					Tokens.push_back(std::make_shared<Token>(ToStr(ch), Token::token_comma));
					continue;
				case L' ':
				case L'\t':
					AddText(Tokens);
					continue;
			}
		}

		Text.push_back(ch);
	} while (true);

	AddText(Tokens);

	if (inQuote)
		throw EraCMDException(CMDERROR_MISSING_END_QUOTE);
}

