/**
 * @file FilesReplace.cpp
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "FilesReplace.h"

#include <Common/era_api/err_codes.h>
#include "EracmdExceptions.h"
#include "FileLoader.h"

FilesReplace::FilesReplace(const std::wstring &input, const uint32_t maxLevel, const uint32_t currentLevel, std::set<std::wstring> &ignoreFiles)
:
	input(input),
	maxLevel(maxLevel),
	currentLevel(currentLevel),
	ignoreFiles(ignoreFiles),
	mark(L'@'),
	quote(L'"')
{
	std::wostringstream s;
	s << mark << quote;
	importantChars = s.str();
}

std::wstring FilesReplace::ReplaceAndGetResult()
{
	ResetStream(output);
	ResetStream(filePath);

	inFilePath = false;
	inQuotes = false;

	ResetPosition();

	while (! IsAtEnd())
	{
		currentPartBegin = pos;

		GoToNextImportantChar();
		AppendRemainingInput();

		if (! IsAtEnd())
		{
			if (IsAtMark())
			{
				if (inQuotes)
				{
					AppendCurrentChar();
				}
				else
				{
					if (inFilePath)
					{
						std::wstring path = filePath.str();

						if (currentLevel >= maxLevel)
						{
							throw EraCMDException(CMDERROR_TOO_MANY_LEVELS_IN_FILE_REPLACING);
						}

						if (ignoreFiles.find(path) != ignoreFiles.end())
						{
							throw EraCMDException(CMDERROR_FILE_REPLACING_LOOP_DETECTED);
						}

						ResetStream(filePath);

						ProcessFile(path);
					}

					inFilePath = !inFilePath;
				}
			}
			else if (IsAtQuote())
			{
				AppendCurrentChar();

				inQuotes = !inQuotes;
			}
		}

		Advance();
	}

	if (inFilePath)
	{
		throw EraCMDException(CMDERROR_FILE_REPLACING_MISSING_END);
	}

	return output.str();
}

void FilesReplace::ResetPosition()
{
	if (input.size() > 0)
	{
		pos = 0;
	}
	else
	{
		pos = std::wstring::npos;
	}
}

void FilesReplace::GoToNextImportantChar()
{
	pos = input.find_first_of(importantChars, pos);
}

void FilesReplace::Advance()
{
	if (pos < (input.size() - 1))
	{
		++pos;
	}
	else
	{
		pos = std::wstring::npos;
	}
}

std::wostringstream &FilesReplace::GetCurrentOutput()
{
	if (inFilePath)
	{
		return filePath;
	}
	else
	{
		return output;
	}
}

void FilesReplace::AppendRemainingInput()
{
	GetCurrentOutput() << input.substr(currentPartBegin, pos - currentPartBegin);
}

void FilesReplace::AppendCurrentChar()
{
	GetCurrentOutput() << input[pos];
}

bool FilesReplace::IsAtEnd() const
{
	return (pos == std::wstring::npos);
}

bool FilesReplace::IsAtMark() const
{
	return (input[pos] == mark);
}

bool FilesReplace::IsAtQuote() const
{
	return (input[pos] == quote);
}

void FilesReplace::ProcessFile(const std::wstring &path)
{
	FileLoader fileLoader(path);
	std::wstring fileContents_orig = fileLoader.Load();

	std::set<std::wstring> ignoreFiles_new = ignoreFiles;
	ignoreFiles_new.insert(path);
	FilesReplace replace(fileContents_orig, maxLevel, currentLevel + 1, ignoreFiles_new);
	std::wstring fileContents_replaced = replace.ReplaceAndGetResult();

	fileContents_replaced = TransformFileContent(fileContents_replaced);

	output << fileContents_replaced;
}

void FilesReplace::ResetStream(std::wostringstream &stream)
{
	stream.str(L"");
	stream.clear();
}

std::wstring FilesReplace::TransformFileContent(const std::wstring &txt)
{
	// take non-empty lines and connect them with comma
	std::wstringstream src(txt);
	std::wstring line;
	std::wstring result;
	while (std::getline(src, line, L'\n'))
	{
		// line is nonempty?
		bool nonEmpty = false;
		for (size_t i = 0; i < line.length(); ++i)
		{
			if (line[i]>L' ')
			{
				nonEmpty = true;
				break;
			}
		}
		if (nonEmpty)
		{
			// remove 0xD at the end
			if (*line.rbegin() == L'\r')
				line.pop_back();
			if (!result.empty())
				result += L",";
			result += line;
		}
	}
	return result;
}

