/**
 * @file Autocomplete.cpp
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Autocomplete.h"

#include "../flags/FlagsRegistry.h"
#include "../util/stringUtils.h"
#include "../EracmdExceptions.h"

extern FlagsRegistry gFlagsRegistry;

Autocomplete::Autocomplete(ScriptInterpreter &scriptInterpreter) : scriptInterpreter(scriptInterpreter)
{
	Reset();
}

void Autocomplete::Reset()
{
	inputLength = 0;
	lastIndex = -1;
	completions.clear();
}

bool Autocomplete::IsReset()
{
	return lastIndex == -1;
}


int Autocomplete::GetNext(const std::wstring &commandLine, std::wstring &completion)
{
	if (lastIndex == -1)
		Build(commandLine);

	if (!completions.empty())
	{
		lastIndex = (lastIndex + 1) % completions.size();
		completion = completions[lastIndex];
	}

	return inputLength;
}


void Autocomplete::Build(const std::wstring &commandLine)
{
	Reset();

	inputLength = (int) commandLine.length();

	std::wstring lineLocked = commandLine; // this part of commandline is complete
	std::wstring lineOpened; // uncomplete word
	for (int i = inputLength - 1; i >= 0; i--)
	{
		WCHAR w = *lineLocked.rbegin();
		if (w >= L'A' && w <= L'Z') w += L'a' - L'A';
		if ((w >= L'0' && w <= L'9') || (w >= L'a' && w <= L'z') || w == L'_' || w == L'-' || w == L'!')
		{
			lineOpened = w + lineOpened;
			lineLocked.pop_back();
		}
		else break;
	}


	OutputDebugStringW(L"building autocomplete for ");
	OutputDebugString(lineOpened.c_str());
	OutputDebugStringW(L"\n");

	BuildCommandAutocomplete(lineLocked, lineOpened);
	BuildFlagsAutocomplete(lineOpened);

	// complete command arguments
	AutocompleteCollector collector(lineOpened, completions);
	try
	{
		scriptInterpreter.ExecuteScript(commandLine, false, &collector);
	}
	catch (const EraCMDException &)
	{
		// these are OK
	}
	catch (const std::exception &e)
	{
		throw e;
	}

	std::sort(completions.begin(), completions.end());

	if (!lineOpened.empty())
	{
		for (size_t i = 0; i < completions.size(); ++i)
		{
			completions[i].erase(0, lineOpened.length());
		}
	}
}







void Autocomplete::BuildCommandAutocomplete(const std::wstring &lineLocked, const std::wstring &lineOpened)
{
	// locked words
	std::vector<std::wstring> lockedWords;
	SplitToWords(lineLocked, L' ', lockedWords);

	// remove extra spaces (there should be no tab, as tab is used for autocomplete)
	std::wstring commandLineNormalized = lineLocked + lineOpened;
	while (ReplaceAllSubstrings(commandLineNormalized, L"  ", L" "));
	while (commandLineNormalized.length() && commandLineNormalized[0] == L' ') commandLineNormalized.erase(0, 1);

	CommandList *list = scriptInterpreter.GetCommandList();
	std::vector<std::wstring> allCommands;
	list->GetCommandStrings(allCommands);

	for (size_t i = 0; i < allCommands.size(); ++i)
	{
		if (_wcsnicmp(allCommands[i].c_str(), commandLineNormalized.c_str(), commandLineNormalized.length()) == 0)
		{
			if (lockedWords.empty())
				completions.push_back(allCommands[i]);
			else // skip locked words
			{
				std::vector<std::wstring> words2;
				SplitToWords(allCommands[i], L' ', words2);
				std::wstring result;
				for (size_t j = lockedWords.size(); j<words2.size(); ++j)
				{
					if (!result.empty()) result += L" ";
					result += words2[j];
				}
				completions.push_back(result);
			}
		}
	}
}

void Autocomplete::BuildFlagsAutocomplete(const std::wstring &lineOpened)
{
	if (lineOpened.empty() || lineOpened[0] != L'-')
		return;

	const FlagsRegistry::Flags &flags = gFlagsRegistry.GetSupportedFlags();

	for (FlagsRegistry::Flags::const_iterator it = flags.begin(); it != flags.end(); ++it)
	{
		std::wstring flag = L"-" + (*it)->GetName();
		if (_wcsnicmp(flag.c_str(), lineOpened.c_str(), lineOpened.length()) == 0)
		{
			completions.push_back(flag);
		}
	}
}






