/**
 * @file Command_Group.cpp
 * @author Copyright (c) 2014 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Command_Group.h"

#include <Common/scope_exit.h>
#include <Common/era_api/err_codes.h>
#include "../util/stringUtils.h"
#include "../EracmdExceptions.h"

Command_Group::Command_Group(CommandFlagsStack &commandFlagsStack)
	: commandFlagsStack(commandFlagsStack)
{
}

size_t BuildGroupTree(const std::vector<StringVector> &rows, std::vector<StringVector> &treeRows, bool staticGroups, size_t i0, bool prettyFormat)
{
	#define SEPARATOR_LOCALGROUPNAME L'/'
	#define TREE_BRANCH_STR L"|__ "
	#define TREE_BRANCH_INDENT L"    "

	std::wstring staticStr = prettyFormat ? LoadStringFromResource(IDS_GROUP_TYPE_STATIC) : L"static";
	std::wstring parametricStr = prettyFormat ? LoadStringFromResource(IDS_GROUP_TYPE_PARAMETRIC) : L"parametric";

	size_t insertedRows = 0;
	for (size_t i = 0; i < rows.size(); ++i)
	{
		if ((staticGroups && rows[i][3]!=staticStr) || (!staticGroups && rows[i][3]!=parametricStr))
			continue;
		size_t nestingLevel = 0;
		if (i > 0)
		{
			size_t iPrev = i-1;
			do
			{
				if ((staticGroups && rows[iPrev][3]==staticStr) || (!staticGroups && rows[iPrev][3]==parametricStr))
				{
					std::wstring toFind = rows[iPrev][1] + SEPARATOR_LOCALGROUPNAME;
					if (rows[i][1].compare(0, toFind.length(), toFind) == 0)
						++nestingLevel;
				}
			}
			while (iPrev-- > 0);
		}

		std::wostringstream newName;
		for (size_t j = 0; j < nestingLevel; ++j)
			newName << TREE_BRANCH_INDENT;
		newName << TREE_BRANCH_STR;

		size_t sepPos;
		if ((sepPos = rows[i][1].rfind(SEPARATOR_LOCALGROUPNAME)) != std::wstring::npos)
		{
			if (sepPos < rows[i][1].length())
				newName << rows[i][1].substr(sepPos+1);
		}
		else
			newName << rows[i][1];

		if (rows[i][2].length() > 0)
			newName << L" (" << rows[i][2] << L')';

		treeRows[i0+insertedRows].push_back(rows[i][0]);
		treeRows[i0+insertedRows].push_back(newName.str());
		++insertedRows;
	}

	return insertedRows;
}

void Command_Group::Execute(elementArray &elements, OutputHandler &out)
{
	std::shared_ptr<Element> treeArg = GetArgument(0);

	Era_GroupQuery *groupQuery = era_GroupQuery_new();
	if (treeArg)
		era_GroupQuery_SetDisplayType(groupQuery, ERA_GROUP_DISPLAY_TYPE__TREE);
	else
		era_GroupQuery_SetDisplayType(groupQuery, ERA_GROUP_DISPLAY_TYPE__TABLE);

	scope_exit se_groupQuery(std::bind(era_GroupQuery_delete, groupQuery));
	Era_GroupTableReader *groupTableReader;
	uint32_t result_openQuery = era_OpenGroupQuery(groupQuery, &groupTableReader);
	if (result_openQuery != 0)
	{
		throw EraCMDException_FromApi(result_openQuery);
	}

	scope_exit se_tableReader(std::bind(era_GroupTableReader_delete, groupTableReader));
	scope_exit_discard_return<DWORD> se_openQuery(std::bind(era_CloseGroupQuery, groupTableReader));

	StringVector columns;
	const Era_TableReader *tableReader = era_GroupTableReader_GetTableReader(groupTableReader);
	uint32_t columnsCount = era_TableReader_GetColumnsCount(tableReader);
	columns.resize(columnsCount);
	for (uint32_t i = 0; i < columnsCount; ++i)
	{
		const Era_ColumnInfo *columnInfo = era_TableReader_GetColumn(tableReader, i);
		columns[i] = era_ColumnInfo_GetHeader(columnInfo);
	}

	StringVector prettyColumns;
	prettyColumns.resize(columnsCount);
	for (uint32_t i = 0; i < columnsCount; ++i)
	{
		const Era_ColumnInfo *columnInfo = era_TableReader_GetColumn(tableReader, i);
		prettyColumns[i] = era_ColumnInfo_GetPrettyHeader(columnInfo);
	}
	std::vector<StringVector> rows;
	StringVector row;

	bool readingComplete = false;
	bool prettyFormat = commandFlagsStack.CurrentFlags().fieldFormatting == CommandFlags::FIELD_PRETTY;
	do
	{
		Era_TableRow *tableRow;
		uint32_t result_readRow = era_ReadGroupRow(groupTableReader, &tableRow);
		if (result_readRow != 0)
		{
			throw EraCMDException_FromApi(result_readRow);
		}

		if (tableRow != NULL)
		{
			scope_exit se_tableRow(std::bind(era_TableRow_delete, tableRow));
			if (prettyFormat)
			{
				uint32_t prettyFieldsCount = era_TableRow_GetPrettyFieldsCount(tableRow);
				row.resize(prettyFieldsCount);
				for (uint32_t i = 0; i < prettyFieldsCount; ++i)
				{
					row[i] = era_TableRow_GetPrettyField(tableRow, i);;
				}
			}
			else
			{
				uint32_t fieldsCount = era_TableRow_GetFieldsCount(tableRow);
				row.resize(fieldsCount);
				for (uint32_t i = 0; i < fieldsCount; ++i)
				{
					row[i] = era_TableRow_GetField(tableRow, i);;
				}
			}

			rows.push_back(std::move(row));
		}
		else
		{
			readingComplete = true;
		}
	} while (! readingComplete);

	if (treeArg)
	{
		#define N_GROUP_INFO_COLUMNS 4
		#define N_GROUP_TREE_COLUMNS 2
		std::vector<StringVector> treeRows(rows.size() + 2);
		StringVector treeRoot(N_GROUP_TREE_COLUMNS);
		treeRoot[0] = L"";
		treeRoot[1] = LoadStringFromResource(IDS_STATIC_GROUPS);
		treeRows[0] = treeRoot;
		size_t insertedItems = BuildGroupTree(rows, treeRows, true, 1, prettyFormat);

		treeRoot[0] = L"";
		treeRoot[1] = LoadStringFromResource(IDS_PARAMETRIC_GROUPS);
		treeRows[insertedItems + 1] = treeRoot;
		BuildGroupTree(rows, treeRows, false, insertedItems + 2, prettyFormat);

		assert(columns.size() >= N_GROUP_INFO_COLUMNS);
		columns.resize(N_GROUP_TREE_COLUMNS);
		prettyColumns.resize(N_GROUP_TREE_COLUMNS);
		TableReader_Const tableReader2(columns, prettyColumns, treeRows);
		out.Print(tableReader2);
	}
	else
	{
		TableReader_Const tableReader2(columns, prettyColumns, rows);
		out.Print(tableReader2);
	}
}
