/**
 * @file Command_ServerConfig.cpp
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Command_ServerConfig.h"

#include <Common/era_api/era_api.h>
#include <Common/era_api/err_codes.h>
#include <Common/era_api/err_lookup.h>
#include <Common/scope_exit.h>
#include "../util/stringUtils.h"
#include "../ConsoleKeyboardInput.h"
#include "../Element.h"
#include "../EracmdExceptions.h"
#include "../Token.h"

Command_ServerConfig::Command_ServerConfig()
:
	Action(SHOW_SERVER_CFG), scriptInterpreter(NULL)
{
}

Command_ServerConfig::Command_ServerConfig(ServerConfigAction Action, ScriptInterpreter *scriptInterpreter /* = NULL */)
:
	Action(Action), scriptInterpreter(scriptInterpreter)
{
}

unsigned int Command_ServerConfig::GetHelpFullId()
{
	switch (Action)
	{
		case GET_SERVER_CFG:
				return IDS_HELP_SERVERCFG_GET;
		case PUT_SERVER_CFG:
				return IDS_HELP_SERVERCFG_PUT;
		case SET_SERVER_CFG:
				return IDS_HELP_SERVERCFG_SET;
		case SETPWD_SERVER_CFG:
				return IDS_HELP_SERVERCFG_SETPWD;
		case LIST_SERVER_CFG:
				return IDS_HELP_SERVERCFG_LIST;
	}
	ASSERT(FALSE); // unknown Action!
	return 0;
}

unsigned int Command_ServerConfig::GetHelpShortId()
{
	switch (Action)
	{
		case GET_SERVER_CFG:
				return IDS_HELP_SERVERCFG_GET_SHORT;
		case PUT_SERVER_CFG:
				return IDS_HELP_SERVERCFG_PUT_SHORT;
		case SET_SERVER_CFG:
				return IDS_HELP_SERVERCFG_SET_SHORT;
		case SETPWD_SERVER_CFG:
				return IDS_HELP_SERVERCFG_SETPWD_SHORT;
		case LIST_SERVER_CFG:
				return IDS_HELP_SERVERCFG_LIST_SHORT;
	}
	ASSERT(FALSE); // unknown Action!
	return 0;
}

unsigned int Command_ServerConfig::GetHelpExampleId()
{
	switch (Action)
	{
		case GET_SERVER_CFG:
				return IDS_HELP_SERVERCFG_GET_EXAMPLE;
		case PUT_SERVER_CFG:
				return IDS_HELP_SERVERCFG_PUT_EXAMPLE;
		case SET_SERVER_CFG:
				return IDS_HELP_SERVERCFG_SET_EXAMPLE;
		case SETPWD_SERVER_CFG:
				return IDS_HELP_SERVERCFG_SETPWD_EXAMPLE;
		case LIST_SERVER_CFG:
				return IDS_HELP_SERVERCFG_LIST_EXAMPLE;
	}
	ASSERT(FALSE); // unknown Action!
	return 0;
}

void Command_ServerConfig::Execute(elementArray &elements, OutputHandler &out)
{
	std::shared_ptr<Element> whatArg = GetArgument(0);

	DWORD rc = 0;
	switch (Action)
	{
		case GET_SERVER_CFG:
				{
					std::wstring file = whatArg->GetString();
					rc = era_GetServerConfiguration(file.c_str());
					break;
				}
		case PUT_SERVER_CFG:
				{
					std::wstring file = whatArg->GetString();
					rc = era_UploadServerConfiguration(file.c_str());
					break;
				}
		case SET_SERVER_CFG:
				{
					if (whatArg->type != Element::element_word)
						throw EraCMDException(CMDERROR_INVALID_PARAMETER, whatArg->GetString().c_str());

					std::shared_ptr<ElementWord> word = std::static_pointer_cast<ElementWord>(whatArg);
					if (word->tokenList.size() != 3 || word->tokenList[1]->type != Token::token_equals)
						throw EraCMDException(CMDERROR_INVALID_COMMAND_SYNTAX);

					std::wstring name = word->tokenList[0]->text;
					std::wstring value = word->tokenList[2]->text;

					rc = era_ServerConfigurationSetValue(name.c_str(), value.c_str());
				}
				break;
		case SETPWD_SERVER_CFG:
				{
					if (whatArg->type != Element::element_word)
						throw EraCMDException(CMDERROR_INVALID_PARAMETER, whatArg->GetString().c_str());

					std::shared_ptr<ElementWord> word = std::static_pointer_cast<ElementWord>(whatArg);
					if (word->tokenList.size() != 1 || word->tokenList[0]->type != Token::token_text)
						throw EraCMDException(CMDERROR_INVALID_COMMAND_SYNTAX);

					std::wstring name = word->tokenList[0]->text;

					ConsoleKeyboardInput keyInput(*scriptInterpreter, true);
					std::wstring passwordMessage = LoadStringFromResource(IDS_ENTER_PASSWORD);
					std::wcout << passwordMessage << L" ";
					keyInput.GetInput();
					std::wstring password = keyInput.GetLine();
					std::wcout << std::endl;

					rc = era_ServerConfigurationSetValue(name.c_str(), password.c_str());
				}
				break;
		case LIST_SERVER_CFG:
				{
					Era_ServerConfigurationList *list;
					uint32_t rc = era_GetServerConfigurationList(&list);
					if	(rc != 0)
						throw EraCMDException_FromApi(rc);
					scope_exit se_serverConfigurationList(std::bind(era_ServerConfigurationList_delete, list));

					StringVector header(2);
					header[0] = L"setting_name";
					header[1] = L"setting_description";

					StringVector prettyHeader(2);
					prettyHeader[0] = LoadStringFromResource(IDS_SETTING_NAME);
					prettyHeader[1] = LoadStringFromResource(IDS_SETTING_DESCRIPTION);

					std::vector<StringVector> rows;
					uint32_t itemsCount = era_ServerConfigurationList_GetItemsCount(list);
					for (uint32_t i = 0; i < itemsCount; ++i)
					{
						const Era_ServerConfigurationItem *item = era_ServerConfigurationList_GetItem(list, i);
						StringVector row(2);
						row[0] = era_ServerConfigurationItem_GetName(item);
						row[1] = era_ServerConfigurationItem_GetDescription(item);
						rows.push_back(row);
					}

					std::sort(rows.begin(), rows.end(), [](StringVector& v1, StringVector& v2) -> bool
					{
						return (_wcsicmp(v1[0].c_str(), v2[0].c_str()) < 0);
					});

					TableReader_Const reader(header, prettyHeader, rows);
					out.Print(reader);

					return;
				}
				break;
		default:
				ASSERT(FALSE); // unknown Action!
	}

	if (rc != ERAERROR_SUCCESS)
	{
		throw EraCMDException_FromApi(rc);
	}
}

