/**
 * @file Command_TaskScan.cpp
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "Command_TaskScan.h"

#include <Common/era_api/era_api.h>
#include <Common/era_api/err_codes.h>
#include <Common/scope_exit.h>
#include "../util/apiUtils.h"
#include "../util/stringUtils.h"
#include "../Element.h"
#include "../EracmdExceptions.h"

void Command_TaskScan::Execute(elementArray &elements, OutputHandler &out)
{
	Era_TaskParams *params = era_TaskParams_new();
	scope_exit se_query(std::bind(era_TaskParams_delete, params));

	FillTaskParams(0, *params);

	Era_ClientList *clients = ElementToClientList(GetArgument(0));
	scope_exit se_clients(std::bind(era_ClientList_delete, clients));

	bool windows = true;
	bool linux3 = true;
	bool linux = true;
	bool mobile = true;
	FillExcludeBools(5, windows, linux3, linux, mobile);

	Era_ScanParams_Windows *params_windows = NULL;
	scope_exit se_params_windows(std::bind(era_ScanParams_Windows_delete, std::ref(params_windows)));
	if (windows)
	{
		params_windows = era_ScanParams_Windows_new();

		FillParams_Windows(6, *params_windows);
	}

	Era_ScanParams_Linux3 *params_linux3 = NULL;
	scope_exit se_params_linux3(std::bind(era_ScanParams_Linux3_delete, std::ref(params_linux3)));
	if (linux3)
	{
		params_linux3 = era_ScanParams_Linux3_new();

		FillParams_Linux3(11, *params_linux3);
	}

	Era_ScanParams_Linux *params_linux = NULL;
	scope_exit se_params_linux(std::bind(era_ScanParams_Linux_delete, std::ref(params_linux)));
	if (linux)
	{
		params_linux = era_ScanParams_Linux_new();

		FillParams_Linux(13, *params_linux);
	}

	Era_ScanParams_Mobile *params_mobile = NULL;
	scope_exit se_params_mobile(std::bind(era_ScanParams_Mobile_delete, std::ref(params_mobile)));
	if (mobile)
	{
		params_mobile = era_ScanParams_Mobile_new();

		FillParams_Mobile(16, *params_mobile);
	}

	unsigned int randomDelay_maxSeconds = 0;
	if (std::shared_ptr<Element> element = GetArgument(18))
	{
		randomDelay_maxSeconds = 60 * ToPositiveNumber(element->GetString());
	}

	uint32_t taskId = -1;
	uint32_t result_taskScan = era_Task_scan(&taskId, params_windows, params_linux3, params_linux, params_mobile, randomDelay_maxSeconds, clients, params);
	if (result_taskScan != 0)
	{
		throw EraCMDException_FromApi(result_taskScan);
	}

	StringVector header(1);
	header[0] = L"task_id";
	StringVector prettyHeader(1);
	prettyHeader[0] = LoadStringFromResource(IDS_TASK_ID);
	std::vector<StringVector> rows;
	StringVector data(1);
	data[0] = std::to_wstring((long long)taskId);
	rows.push_back(data);
	TableReader_Const reader(header, prettyHeader, rows);
	out.Print(reader);
}

void Command_TaskScan::FillParams_Windows(const int argNumOffset, Era_ScanParams_Windows &params)
{
	if (std::shared_ptr<Element> element = GetArgument(argNumOffset + 0))
	{
		const std::wstring elementStr = element->GetString();

		Era_ScanProfile profile;
		if (KeywordToScanProfile(elementStr, profile))
		{
			era_ScanParams_Windows_SetProfile(&params, profile);
		}
		else
		{
			era_ScanParams_Windows_SetProfile_userDefined(&params, elementStr.c_str());
		}
	}

	if (std::shared_ptr<Element> element = GetArgument(argNumOffset + 1))
	{
		typedef std::map<std::wstring, Era_Target_Windows> KeywordToWindowsTarget;
		KeywordToWindowsTarget keywordToWindowsTarget;
		keywordToWindowsTarget[L"!Memory"] = ERA_TARGET_WINDOWS__MEMORY;
		keywordToWindowsTarget[L"!RemovableDrivesBoot"] = ERA_TARGET_WINDOWS__REMOVABLE_DRIVES_BOOT;
		keywordToWindowsTarget[L"!RemovableDrives"] = ERA_TARGET_WINDOWS__REMOVABLE_DRIVES;
		keywordToWindowsTarget[L"!LocalDrivesBoot"] = ERA_TARGET_WINDOWS__LOCAL_DRIVES_BOOT;
		keywordToWindowsTarget[L"!LocalDrives"] = ERA_TARGET_WINDOWS__LOCAL_DRIVES;
		keywordToWindowsTarget[L"!RemoteDrives"] = ERA_TARGET_WINDOWS__REMOTE_DRIVES;
		keywordToWindowsTarget[L"!AllDrivesBoot"] = ERA_TARGET_WINDOWS__ALL_DRIVES_BOOT;
		keywordToWindowsTarget[L"!AllDrives"] = ERA_TARGET_WINDOWS__ALL_DRIVES;

		std::vector<const ElementWord *> targets = element->GetWords();
		for (std::vector<const ElementWord *>::const_iterator it = targets.begin(); it != targets.end(); ++it)
		{
			std::wstring targetStr = (*it)->GetString();

			bool keywordFound = false;
			for (KeywordToWindowsTarget::const_iterator it = keywordToWindowsTarget.begin(); it != keywordToWindowsTarget.end(); ++it)
			{
				if (::_wcsicmp(targetStr.c_str(), it->first.c_str()) == 0)
				{
					era_ScanParams_Windows_AddTarget(&params, it->second);
					keywordFound = true;
					break;
				}
			}

			if (! keywordFound)
			{
				era_ScanParams_Windows_AddTarget_userDefinedPath(&params, targetStr.c_str());
			}
		}
	}
	else
	{
		era_ScanParams_Windows_AddTarget(&params, ERA_TARGET_WINDOWS__MEMORY);
		era_ScanParams_Windows_AddTarget(&params, ERA_TARGET_WINDOWS__LOCAL_DRIVES_BOOT);
		era_ScanParams_Windows_AddTarget(&params, ERA_TARGET_WINDOWS__LOCAL_DRIVES);
	}

	if (std::shared_ptr<Element> element = GetArgument(argNumOffset + 2))
	{
		if (::ToBool(element->GetString()))
		{
			era_ScanParams_Windows_SetNoCleaning(&params);
		}
	}

	if (std::shared_ptr<Element> element = GetArgument(argNumOffset + 3))
	{
		if (::ToBool(element->GetString()))
		{
			era_ScanParams_Windows_SetShutdownComputerAfterScan(&params);
		}
	}

	if (std::shared_ptr<Element> element = GetArgument(argNumOffset + 4))
	{
		if (::ToBool(element->GetString()))
		{
			era_ScanParams_Windows_SetAllowShutdownCancel(&params);
		}
	}
}

void Command_TaskScan::FillParams_Linux3(const int argNumOffset, Era_ScanParams_Linux3 &params)
{
	if (std::shared_ptr<Element> element = GetArgument(argNumOffset + 0))
	{
		std::vector<const ElementWord *> targets = element->GetWords();
		for (std::vector<const ElementWord *>::const_iterator it = targets.begin(); it != targets.end(); ++it)
		{
			std::wstring targetStr = (*it)->GetString();

			era_ScanParams_Linux3_AddTarget_userDefinedPath(&params, targetStr.c_str());
		}
	}
	else
	{
		era_ScanParams_Linux3_AddTarget_userDefinedPath(&params, L"/");
	}

	if (std::shared_ptr<Element> element = GetArgument(argNumOffset + 1))
	{
		if (::ToBool(element->GetString()))
		{
			era_ScanParams_Linux3_SetNoCleaning(&params);
		}
	}
}

void Command_TaskScan::FillParams_Linux(const int argNumOffset, Era_ScanParams_Linux &params)
{
	if (std::shared_ptr<Element> element = GetArgument(argNumOffset + 0))
	{
		const std::wstring elementStr = element->GetString();

		Era_ScanProfile profile;
		if (KeywordToScanProfile(elementStr, profile))
		{
			era_ScanParams_Linux_SetProfile(&params, profile);
		}
		else
		{
			era_ScanParams_Linux_SetProfile_userDefined(&params, elementStr.c_str());
		}
	}

	if (std::shared_ptr<Element> element = GetArgument(argNumOffset + 1))
	{
		std::vector<const ElementWord *> targets = element->GetWords();
		for (std::vector<const ElementWord *>::const_iterator it = targets.begin(); it != targets.end(); ++it)
		{
			std::wstring targetStr = (*it)->GetString();

			era_ScanParams_Linux_AddTarget_userDefinedPath(&params, targetStr.c_str());
		}
	}
	else
	{
		era_ScanParams_Linux_AddTarget_userDefinedPath(&params, L"/");
	}

	if (std::shared_ptr<Element> element = GetArgument(argNumOffset + 2))
	{
		if (::ToBool(element->GetString()))
		{
			era_ScanParams_Linux_SetNoCleaning(&params);
		}
	}
}

void Command_TaskScan::FillParams_Mobile(const int argNumOffset, Era_ScanParams_Mobile &params)
{
	if (std::shared_ptr<Element> element = GetArgument(argNumOffset + 0))
	{
		typedef std::map<std::wstring, Era_Target_Mobile> KeywordToMobileTarget;
		KeywordToMobileTarget keywordToMobileTarget;
		keywordToMobileTarget[L"!All"] = ERA_TARGET_MOBILE__ALL;

		std::vector<const ElementWord *> targets = element->GetWords();
		for (std::vector<const ElementWord *>::const_iterator it = targets.begin(); it != targets.end(); ++it)
		{
			std::wstring targetStr = (*it)->GetString();

			bool keywordFound = false;
			for (KeywordToMobileTarget::const_iterator it = keywordToMobileTarget.begin(); it != keywordToMobileTarget.end(); ++it)
			{
				if (::_wcsicmp(targetStr.c_str(), it->first.c_str()) == 0)
				{
					era_ScanParams_Mobile_AddTarget(&params, it->second);
					keywordFound = true;
					break;
				}
			}

			if (! keywordFound)
			{
				era_ScanParams_Mobile_AddTarget_userDefinedPath(&params, targetStr.c_str());
			}
		}
	}
	else
	{
		era_ScanParams_Mobile_AddTarget(&params, ERA_TARGET_MOBILE__ALL);
	}

	if (std::shared_ptr<Element> element = GetArgument(argNumOffset + 1))
	{
		if (::ToBool(element->GetString()))
		{
			era_ScanParams_Mobile_SetNoCleaning(&params);
		}
	}
}

bool Command_TaskScan::KeywordToScanProfile(const std::wstring elementStr, Era_ScanProfile &profile)
{
	typedef std::map<std::wstring, Era_ScanProfile> KeywordToScanProfile;
	KeywordToScanProfile keywordToScanProfile;
	keywordToScanProfile[L"!InDepthScan"] = ERA_SCAN_PROFILE__IN_DEPTH_SCAN;
	keywordToScanProfile[L"!MyProfile"] = ERA_SCAN_PROFILE__MY_PROFILE;
	keywordToScanProfile[L"!SmartScan"] = ERA_SCAN_PROFILE__SMART_SCAN;
	keywordToScanProfile[L"!ContextMenuScan"] = ERA_SCAN_PROFILE__CONTEXT_MENU_SCAN;

	for (KeywordToScanProfile::const_iterator it = keywordToScanProfile.begin(); it != keywordToScanProfile.end(); ++it)
	{
		if (::_wcsicmp(elementStr.c_str(), it->first.c_str()) == 0)
		{
			profile = it->second;
			return true;
		}
	}

	return false;
}
