/**
 * @file FlagsRegistry.h
 * @author Copyright (c) 2014 ESET, spol. s r.o.
 */

#pragma once

#include "CommandFlags.h"

class FlagBase
{

public:
	virtual ~FlagBase() {};

	virtual const std::wstring &GetName() const = 0;
	virtual std::wstring GetFrom(const CommandFlags &flags) const = 0;
	virtual void SetTo(CommandFlags &flags, const std::wstring &value) const = 0;

};

template<typename T>
class Flag : public FlagBase
{

public:
	typedef T CommandFlags::*CommandFlagsMember;
	typedef std::function<const std::wstring(const T &)> FromConversionFunction;
	typedef std::function<T(const std::wstring &)> ToConversionFunction;
	typedef std::function<void(const T &)> ApplyFunction;

public:
	Flag(const std::wstring &name, const CommandFlagsMember member, const FromConversionFunction fromConversionFunction, const ToConversionFunction toConversionFunction, const ApplyFunction applyFunction = ApplyFunction())
	:
		name(name),
		member(member),
		fromConversionFunction(fromConversionFunction),
		toConversionFunction(toConversionFunction),
		applyFunction(applyFunction)
	{}

	virtual const std::wstring &GetName() const override
	{
		return name;
	}

	virtual std::wstring GetFrom(const CommandFlags &flags) const override
	{
		return fromConversionFunction(flags.*member);
	}

	virtual void SetTo(CommandFlags &flags, const std::wstring &value) const override
	{
		flags.*member = toConversionFunction(value);
		if (applyFunction) {
			applyFunction(flags.*member);
		}
	}

private:
	std::wstring name;
	CommandFlagsMember member;

	FromConversionFunction fromConversionFunction;
	ToConversionFunction toConversionFunction;
	ApplyFunction applyFunction;

};

class FlagsRegistry
{

public:
	typedef std::vector<std::shared_ptr<FlagBase> > Flags;

public:
	FlagsRegistry();

	const Flags &GetSupportedFlags() const;
	std::shared_ptr<FlagBase> GetFlag(const std::wstring &name);
	bool IsFlag(const std::wstring &str, const std::wstring &prefix = L"") const;

private:

	Flags flags;

};

