/**
 * @file OutputFormatter_Table.cpp
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "OutputFormatter_Table.h"

#include "../EracmdExceptions.h"

OutputFormatter_Table::OutputFormatter_Table(ConsoleColors_Win &consoleColors)
:
	consoleColors(consoleColors)
{
}

DWORD OutputFormatter_Table::PrintClippedString(std::wostream &out, const std::wstring &str, DWORD actualWidth, const TextWidth &textWidth)
{
	if (actualWidth >= consoleWidth)
		return actualWidth;

	size_t strWidth = textWidth.GetWidth(str);
	if (strWidth + actualWidth < consoleWidth)
	{
		out << str;
		return actualWidth + (DWORD) strWidth;
	}

	out << textWidth.AlignTo(str, consoleWidth - actualWidth-1);
	out << consoleColors.GREEN << L">" << consoleColors.DEFAULT;

	return consoleWidth;
}

void OutputFormatter_Table::Print(TableReader &reader, std::wostream &out, const CommandFlags::HeaderType headerType, DWORD screenWidth, const TextWidth &textWidth)
{
	std::vector<std::wstring::size_type> maxLengths;

	consoleWidth = screenWidth;


	StringVector columns;
	if (headerType != CommandFlags::HEADER_NONE)
	{
		columns = (headerType == CommandFlags::HEADER_KEYWORD) ? reader.GetColumns_keyword() : reader.GetColumns_pretty();

		maxLengths.resize(columns.size());

		for (size_t i = 0; i < columns.size(); ++i)
		{
			AddMax(maxLengths, i, textWidth.GetWidth(columns[i]));
		}
	}

	std::vector<StringVector> rows;
	while (reader.HasNext())
	{
		const StringVector row = reader.GetNext();

		maxLengths.resize(row.size());

		for (size_t i = 0; i < row.size(); ++i)
		{
			AddMax(maxLengths, i, textWidth.GetWidth(row[i]));
		}
		rows.push_back(std::move(row));
	}

	if (consoleWidth == 0)
	{
		consoleWidth = 10; // reserve
		for (size_t i = 0; i < maxLengths.size(); ++i)
			consoleWidth += (DWORD) MaxLengthToColumnWidth(maxLengths[i]) + 1;
	}


	if (headerType != CommandFlags::HEADER_NONE)
	{
		PrintSeparator(maxLengths, out, textWidth);
		PrintRow(maxLengths, columns, out, true, consoleColors.DEFAULT, consoleColors.GREEN, consoleColors.GREEN, textWidth);
	}
	PrintSeparator(maxLengths, out, textWidth);

	for (size_t i = 0; i < rows.size(); ++i)
	{
		const StringVector &row = rows[i];
		if (i % 2 == 1)
			PrintRow(maxLengths, row, out, false, consoleColors.DEFAULT_LIGHTBACK, consoleColors.GREEN_LIGHTBACK, consoleColors.GREEN, textWidth);
		else
			PrintRow(maxLengths, row, out, false, consoleColors.DEFAULT, consoleColors.GREEN, consoleColors.GREEN, textWidth);
	}

	PrintSeparator(maxLengths, out, textWidth);

	out << consoleColors.AQUA << rows.size() << consoleColors.DEFAULT << ' ';
	out << ((rows.size() == 1) ? "row" : "rows");
	out << std::endl;
}

void OutputFormatter_Table::PrintSeparator(const std::vector<std::wstring::size_type> &maxLengths, std::wostream &out, const TextWidth &textWidth)
{

	out << consoleColors.GREEN;

	DWORD actualWidth = 0;

	for (size_t i = 0; i < maxLengths.size(); ++i)
	{
		if (i == 0)
		{
			actualWidth = PrintClippedString(out, L"+", actualWidth, textWidth);
		}
		actualWidth = PrintClippedString(out, std::wstring(MaxLengthToColumnWidth(maxLengths[i]), L'-'), actualWidth, textWidth);

		if (i<maxLengths.size()-1)
			actualWidth = PrintClippedString(out, L"+", actualWidth, textWidth);
		else
		{
			if (actualWidth<consoleWidth)
			{
				actualWidth++;
				out << L'+';
			}
		}
	}
	out << consoleColors.DEFAULT;

	if (actualWidth != consoleWidth)
		out << std::endl;
}

void OutputFormatter_Table::PrintRow(const std::vector<std::wstring::size_type> &maxLengths, const StringVector &texts, std::wostream &out, const bool isHeader, const Color &lineColor, const Color &pipeColor, const Color &pipeColorBorder, const TextWidth &textWidth)
{

	DWORD actualWidth = 0;
	for (size_t i = 0; i < maxLengths.size(); ++i)
	{
		if (i == 0)
		{
			out << pipeColorBorder;
			actualWidth = PrintClippedString(out, L"|", actualWidth, textWidth);
			out << lineColor;
		}

		actualWidth = PrintClippedString(out, L" ", actualWidth, textWidth);
		out << (isHeader ? consoleColors.AQUA : lineColor);
		actualWidth = PrintClippedString(out, textWidth.ReplaceVariableLengthCharacters(texts[i]), actualWidth, textWidth);
		out << lineColor;
		actualWidth = PrintClippedString(out, std::wstring(maxLengths[i] - textWidth.GetWidth(texts[i]), L' '), actualWidth, textWidth);
		out << lineColor;
		actualWidth = PrintClippedString(out, L" ", actualWidth, textWidth);
		if (i<maxLengths.size()-1)
		{
			out << pipeColor;
			actualWidth = PrintClippedString(out, L"|", actualWidth, textWidth);
		}
		else
		{
			out << pipeColorBorder;
			if (actualWidth<consoleWidth)
			{
				actualWidth++;
				out << L'|';
			}
		}

		out << lineColor;
	}
	out << consoleColors.DEFAULT;

	if (actualWidth != consoleWidth)
		out << std::endl;
}

StringVector::size_type OutputFormatter_Table::MaxLengthToColumnWidth(const StringVector::size_type maxLength)
{
	return maxLength + 2;
}

void OutputFormatter_Table::AddMax(std::vector<std::wstring::size_type> &maxLengths, const size_t column, const std::wstring::size_type length)
{
	maxLengths[column] = std::max<std::wstring::size_type>(maxLengths[column], length);
}

