/**
 * @file ParamValidation.cpp
 * @author Copyright (c) 2013 ESET, spol. s r.o.
 */

#include "set_nod.h"
#include "ParamValidation.h"

#include <Common/era_api/era_api.h>
#include <Common/era_api/err_codes.h>
#include <Common/scope_exit.h>
#include "../Command.h"
#include "../Element.h"
#include "../EracmdExceptions.h"
#include "../Token.h"
#include "../flags/FlagsRegistry.h"
#include "../util/stringUtils.h"
#include "../ScriptInterpreter.h"
#include "Autocomplete.h"

extern FlagsRegistry gFlagsRegistry;

//////////////////////////////////////////////////////////////////////////
//	BASE CLASS
//////////////////////////////////////////////////////////////////////////

void ParamValidator::ValidateParam(std::shared_ptr<Element> param, Command *currentCommand)
{
	std::vector<const ElementWord *> words = param->GetWords();

	ValidateAllowList(words, param);

	for (size_t i = 0; i < words.size(); ++i)
	{
		ValidateSingleParam(words[i]);
	}
}

void ParamValidator::ValidateAllowList(const std::vector<const ElementWord *> &words, std::shared_ptr<Element> param)
{
	if (words.size() > 1 && allowList == false)
	{
		throw EraCMDException(CMDERROR_LIST_NOT_SUPPORTED, param->GetString().c_str());
	}
}

void ParamValidator::AutoComplete(std::shared_ptr<Element> param, Command *currentCommand, AutocompleteCollector *collector)
{
	if (param && allowList == false && (param->type == Element::element_wordlist || param->openedList == true))
		return; // list not supported

	if (enumValues.empty())
		return;

	if (param && !param->openedList && !param->missingOperand)
	{
		if (param->type == Element::element_wordlist)
		{
			std::shared_ptr<ElementWordList> wlist = std::static_pointer_cast<ElementWordList>(param);
			std::shared_ptr<ElementWord> lastWord = *wlist->wordList.rbegin();
			AutoCompleteWithList(lastWord->GetString(), enumValues, collector);
		}
		else
			AutoCompleteWithList(param->GetString(), enumValues, collector);
	}
	else
		AutoCompleteWithList(L"", enumValues, collector);
}

void ParamValidator::AutoCompleteWithList(const std::wstring &word, const std::vector<std::wstring> &options, AutocompleteCollector *collector)
{
	for (size_t i = 0; i < options.size(); ++i)
	{
		if (options[i].length() >= word.length() && _wcsnicmp(options[i].c_str(), word.c_str(), word.length()) == 0)
			collector->Add(options[i]);
	}
}

void ParamValidator::ValidateSingleParam(const Element *param)
{
	std::wstring paramString = param->GetString();
	for (size_t i = 0; i < enumValues.size(); ++i)
	{
		if (_wcsicmp(paramString.c_str(), enumValues[i].c_str()) == 0)
			return;
	}
	{
		std::wstring options = enumValues[0];
		for (size_t i = 1; i < enumValues.size(); ++i)
			options += L"," + enumValues[i];
		throw EraCMDException(CMDERROR_INVALID_ARGUMENT_VALUE, paramString.c_str(), options.c_str());
	}
}



//////////////////////////////////////////////////////////////////////////
//	OTHER VALIDATORS
//////////////////////////////////////////////////////////////////////////

ParamValidatorEnum::ParamValidatorEnum(std::wstring enumCommaList, bool allowList, bool validate) : ParamValidator(allowList), validate(validate)
{
	SplitToWords(enumCommaList, L',', enumValues);
}

void ParamValidatorEnum::ValidateParam(std::shared_ptr<Element> param, Command *currentCommand)
{
	if (validate)
		return ParamValidator::ValidateParam(param, currentCommand);
	else if (!allowList)
		return ParamValidator::ValidateAllowList(param->GetWords(), param);
}


ParamValidatorEnumConditional::ParamValidatorEnumConditional(int tableNameArgumentIndex, std::wstring enumCommaList, bool allowList, bool validate) 
	:	tableNameArgumentIndex(tableNameArgumentIndex), 
		ParamValidatorEnum(enumCommaList, allowList, validate)
{	
}

void ParamValidatorEnumConditional::ValidateParam(std::shared_ptr<Element> param, Command *currentCommand)
{
	ParamValidatorEnum::ValidateParam(param, currentCommand);
}

void ParamValidatorEnumConditional::AutoComplete(std::shared_ptr<Element> param, Command *currentCommand, AutocompleteCollector *collector)
{
	if (currentCommand->GetArgument(tableNameArgumentIndex)==NULL) return;
	ParamValidatorEnum::AutoComplete(param, currentCommand, collector);
}


ParamValidatorFlag::ParamValidatorFlag(bool allowList, bool addSave) : ParamValidator(allowList)
{
	if (addSave) {
		enumValues.push_back(L"save");
	}

	const FlagsRegistry::Flags &flags = gFlagsRegistry.GetSupportedFlags();
	for (FlagsRegistry::Flags::const_iterator it = flags.begin(); it != flags.end(); ++it)
	{
		enumValues.push_back((*it)->GetName());
	}
}


ParamValidatorNoList::ParamValidatorNoList() : ParamValidator(false)
{
}

ParamValidatorHelp::ParamValidatorHelp(const CommandList &commandList) : ParamValidator(false)
{
	enumValues.push_back(L"commands");
	enumValues.push_back(L"flags");
	enumValues.push_back(L"help");
	commandList.GetCommandStrings(enumValues);
}

void ParamValidatorHelp::ValidateParam( std::shared_ptr<Element> param, Command *currentCommand )
{
	if (!allowList)
		return ParamValidator::ValidateAllowList(param->GetWords(), param);
}

//////////////////////////////////////////////////////////////////////////
//	TABLES
//////////////////////////////////////////////////////////////////////////
ParamValidatorTables::ParamValidatorTables(bool allowList) : ParamValidator(allowList)
{
	Era_TablesInfo *tablesInfo;
	uint32_t result_getTablesInfo = era_GetTablesInfo(&tablesInfo);
	if (result_getTablesInfo != 0)
		throw EraCMDException_FromApi(result_getTablesInfo);

	scope_exit se_tablesInfo(std::bind(era_TablesInfo_delete, tablesInfo));
	uint32_t tableNamesCount = era_TablesInfo_GetTableNamesCount(tablesInfo);
	for (uint32_t i = 0; i < tableNamesCount; ++i)
	{
		enumValues.push_back(era_TablesInfo_GetTableName(tablesInfo, i));
	}
}

//////////////////////////////////////////////////////////////////////////
//	Set server configuration available items
//////////////////////////////////////////////////////////////////////////
ParamValidatorServerCfgItems::ParamValidatorServerCfgItems(bool allowList, bool onlyPwd) : ParamValidator(allowList)
{
	Era_ServerConfigurationList *list;

	uint32_t rc = era_GetServerConfigurationList(&list);
	if	(rc != 0)
		throw EraCMDException_FromApi(rc);

	scope_exit se_serverConfigurationList(std::bind(era_ServerConfigurationList_delete, list));

	uint32_t itemsCount = era_ServerConfigurationList_GetItemsCount(list);
	for (uint32_t i = 0; i < itemsCount; ++i)
	{
		const Era_ServerConfigurationItem *item = era_ServerConfigurationList_GetItem(list, i);
		if (!onlyPwd || era_ServerConfigurationItem_GetType(item) == ERA_SERVER_CFG_TYPE__PASSWORD_VALUE)
			enumValues.push_back(era_ServerConfigurationItem_GetName(item));
	}
};

void ParamValidatorServerCfgItems::ValidateParam( std::shared_ptr<Element> param, Command *currentCommand )
{
	if (!allowList)
		return ParamValidator::ValidateAllowList(param->GetWords(), param);
}

//////////////////////////////////////////////////////////////////////////
//	COLUMNS (columns for table defined in other param)
//////////////////////////////////////////////////////////////////////////

ParamValidatorColumns::ParamValidatorColumns(int tableNameArgumentIndex, bool allowList, bool selectableOnly) : tableNameArgumentIndex(tableNameArgumentIndex), ParamValidator(allowList), selectableOnly(selectableOnly)
{
}

void ParamValidatorColumns::AutoComplete(std::shared_ptr<Element> param, Command *currentCommand, AutocompleteCollector *collector)
{
	enumValues.clear();

	std::wstring tableName = currentCommand->GetArgumentAsString(tableNameArgumentIndex);

	bool isNextInList = (param) && (param->openedList || param->GetWords().size() >= 2);

	if (selectableOnly && (! isNextInList))
	{
		enumValues.push_back(L"*");
		enumValues.push_back(L"count");
	}

	bool wasAllOrCount = false;
	if (selectableOnly && isNextInList)
	{
		if (
			(::_wcsicmp(param->GetWords()[0]->GetString().c_str(), L"*") == 0)
			||
			(::_wcsicmp(param->GetWords()[0]->GetString().c_str(), L"count") == 0)
		)
		{
			wasAllOrCount = true;
		}
	}

	if (! wasAllOrCount)
	{
		Era_TableReader *columnsInfo;
		uint32_t result_getColumnsInfo = era_GetColumnsInfo(tableName.c_str(), &columnsInfo);
		if (result_getColumnsInfo != 0)
			throw EraCMDException_FromApi(result_getColumnsInfo);

		scope_exit se_columnsInfo(std::bind(era_TableReader_delete, columnsInfo));
		uint32_t columnsCount = era_TableReader_GetColumnsCount(columnsInfo);
		for (uint32_t i = 0; i < columnsCount; ++i)
		{
			const Era_ColumnInfo *columnInfo = era_TableReader_GetColumn(columnsInfo, i);
			if ((! selectableOnly) || (!  era_ColumnInfo_IsMultipleValues(columnInfo)))
			{
				enumValues.push_back(era_ColumnInfo_GetHeader(columnInfo));
			}
		}
	}

	ParamValidator::AutoComplete(param, currentCommand, collector);
}

void ParamValidatorColumns::ValidateParam( std::shared_ptr<Element> param, Command *currentCommand )
{
	if (!allowList)
		return ParamValidator::ValidateAllowList(param->GetWords(), param);
}


//////////////////////////////////////////////////////////////////////////
//	SUBSET (when param can be only a subset of other param)
//////////////////////////////////////////////////////////////////////////

ParamValidatorColumnsWithValues::ParamValidatorColumnsWithValues(int tableNameArgumentIndex, bool allowList)
	: tableNameArgumentIndex(tableNameArgumentIndex), ParamValidator(allowList)
{
}

void ParamValidatorColumnsWithValues::AutoComplete(std::shared_ptr<Element> param, Command *currentCommand, AutocompleteCollector *collector)
{
	if (param && allowList == false && (param->type == Element::element_wordlist || param->openedList == true))
		return; // list not supported

	enumValues.clear();

	bool useColumnName = false;
	std::wstring useValueForColumn = L"";

	if (!param || param->openedList)
		useColumnName = true;
	else
	{
		std::shared_ptr<ElementWord> lastWord;
		if (param->type == Element::element_wordlist)
			lastWord = *((ElementWordList&)(*param)).wordList.rbegin();
		else
			lastWord = std::static_pointer_cast<ElementWord>(param);

		if (lastWord)
		{
			if (lastWord->tokenList.size() == 1 && !collector->openWord.empty())
				useColumnName = true;
			else
			if (lastWord->missingOperand && lastWord->tokenList.size() >= 2 && lastWord->tokenList[0]->type == Token::token_text) // values
			{
				useValueForColumn = lastWord->tokenList[0]->text;
			}
			else
			if (lastWord->tokenList.size() >= 3 && lastWord->tokenList[0]->type == Token::token_text && lastWord->tokenList[2]->type == Token::token_text && lastWord->tokenList[lastWord->tokenList.size()-1]->text == collector->openWord) // values
			{
				useValueForColumn = lastWord->tokenList[0]->text;
			}
		}
	}

	if (useColumnName || !useValueForColumn.empty())
	{
		std::wstring tableName = currentCommand->GetArgumentAsString(tableNameArgumentIndex);

		Era_TableReader *columnsInfo;
		uint32_t result_getColumnsInfo = era_GetColumnsInfo(tableName.c_str(), &columnsInfo);
		if (result_getColumnsInfo != 0)
			throw EraCMDException_FromApi(result_getColumnsInfo);

		scope_exit se_columnsInfo(std::bind(era_TableReader_delete, columnsInfo));
		uint32_t columnsCount = era_TableReader_GetColumnsCount(columnsInfo);
		std::vector<StringVector> rows(columnsCount);
		if (useColumnName)
		{
			for (uint32_t i = 0; i < columnsCount; ++i)
			{
				const Era_ColumnInfo *columnInfo = era_TableReader_GetColumn(columnsInfo, i);
				enumValues.push_back(era_ColumnInfo_GetHeader(columnInfo));
			}
		}
		else if (!useValueForColumn.empty())
		{
			for (uint32_t i = 0; i < columnsCount; ++i)
			{
				const Era_ColumnInfo *columnInfo = era_TableReader_GetColumn(columnsInfo, i);
				if (_wcsicmp(era_ColumnInfo_GetHeader(columnInfo), useValueForColumn.c_str()) == 0)
				{
					uint32_t typeValuesCount = era_ColumnInfo_GetTypeValuesCount(columnInfo);
					for (uint32_t j = 0; j < typeValuesCount; ++j)
					{
						enumValues.push_back(era_ColumnInfo_GetTypeValue(columnInfo, j));
					}
				}
			}
		}
		AutoCompleteWithList(collector->openWord, enumValues, collector);
	}
}


void ParamValidatorColumnsWithValues::ValidateParam( std::shared_ptr<Element> param, Command *currentCommand )
{
	if (!allowList)
		return ParamValidator::ValidateAllowList(param->GetWords(), param);
}
